@@ -25,8 +25,8 @@
 #include <glib.h>
 
 #define GDK_MAJOR_VERSION (3)
-#define GDK_MINOR_VERSION (14)
-#define GDK_MICRO_VERSION (12)
+#define GDK_MINOR_VERSION (16)
+#define GDK_MICRO_VERSION (7)
 
 #ifndef _GDK_EXTERN
 #define _GDK_EXTERN extern
@@ -132,6 +132,16 @@
  */
 #define GDK_VERSION_3_14        (G_ENCODE_VERSION (3, 14))
 
+/**
+ * GDK_VERSION_3_16:
+ *
+ * A macro that evaluates to the 3.16 version of GDK, in a format
+ * that can be used by the C pre-processor.
+ *
+ * Since: 3.16
+ */
+#define GDK_VERSION_3_16        (G_ENCODE_VERSION (3, 16))
+
 /* evaluates to the current stable version; for development cycles,
  * this means the next stable target
  */
@@ -316,5 +326,19 @@
 # define GDK_AVAILABLE_IN_3_14                _GDK_EXTERN
 #endif
 
+#if GDK_VERSION_MIN_REQUIRED >= GDK_VERSION_3_16
+# define GDK_DEPRECATED_IN_3_16               GDK_DEPRECATED
+# define GDK_DEPRECATED_IN_3_16_FOR(f)        GDK_DEPRECATED_FOR(f)
+#else
+# define GDK_DEPRECATED_IN_3_16               _GDK_EXTERN
+# define GDK_DEPRECATED_IN_3_16_FOR(f)        _GDK_EXTERN
+#endif
+
+#if GDK_VERSION_MAX_ALLOWED < GDK_VERSION_3_16
+# define GDK_AVAILABLE_IN_3_16                GDK_UNAVAILABLE(3, 16)
+#else
+# define GDK_AVAILABLE_IN_3_16                _GDK_EXTERN
+#endif
+
 #endif  /* __GDK_VERSION_MACROS_H__ */
 
