@@ -29,6 +29,8 @@
 #include "gdkwayland.h"
 #include "gdkprivate-wayland.h"
 
+#include "wm-button-layout-translation.h"
+
 typedef struct _GdkWaylandScreen      GdkWaylandScreen;
 typedef struct _GdkWaylandScreenClass GdkWaylandScreenClass;
 
@@ -96,6 +98,8 @@ struct _GdkWaylandMonitor
   gint          scale;
 };
 
+GType _gdk_wayland_screen_get_type (void);
+
 G_DEFINE_TYPE (GdkWaylandScreen, _gdk_wayland_screen, GDK_TYPE_SCREEN)
 
 static void
@@ -106,6 +110,7 @@ free_monitor (gpointer data)
   if (monitor == NULL)
     return;
 
+  wl_output_destroy (monitor->output);
   g_free (monitor->output_name);
   g_free (monitor->manufacturer);
 
@@ -218,7 +223,9 @@ gdk_wayland_screen_get_monitor_width_mm	(GdkScreen *screen,
 					 gint       monitor_num)
 {
   GdkWaylandScreen *screen_wayland = GDK_WAYLAND_SCREEN (screen);
-  GdkWaylandMonitor *monitor = g_ptr_array_index(screen_wayland->monitors, monitor_num);
+  GdkWaylandMonitor *monitor;
+
+  monitor = g_ptr_array_index (screen_wayland->monitors, monitor_num);
 
   return monitor->width_mm;
 }
@@ -228,7 +235,9 @@ gdk_wayland_screen_get_monitor_height_mm (GdkScreen *screen,
 					  gint       monitor_num)
 {
   GdkWaylandScreen *screen_wayland = GDK_WAYLAND_SCREEN (screen);
-  GdkWaylandMonitor *monitor = g_ptr_array_index(screen_wayland->monitors, monitor_num);
+  GdkWaylandMonitor *monitor;
+
+  monitor = g_ptr_array_index (screen_wayland->monitors, monitor_num);
 
   return monitor->height_mm;
 }
@@ -238,7 +247,9 @@ gdk_wayland_screen_get_monitor_plug_name (GdkScreen *screen,
 					  gint       monitor_num)
 {
   GdkWaylandScreen *screen_wayland = GDK_WAYLAND_SCREEN (screen);
-  GdkWaylandMonitor *monitor = g_ptr_array_index(screen_wayland->monitors, monitor_num);
+  GdkWaylandMonitor *monitor;
+
+  monitor = g_ptr_array_index (screen_wayland->monitors, monitor_num);
 
   return g_strdup (monitor->output_name);
 }
@@ -249,7 +260,9 @@ gdk_wayland_screen_get_monitor_geometry (GdkScreen    *screen,
 					 GdkRectangle *dest)
 {
   GdkWaylandScreen *screen_wayland = GDK_WAYLAND_SCREEN (screen);
-  GdkWaylandMonitor *monitor = g_ptr_array_index(screen_wayland->monitors, monitor_num);
+  GdkWaylandMonitor *monitor;
+
+  monitor = g_ptr_array_index (screen_wayland->monitors, monitor_num);
 
   if (dest)
     *dest = monitor->geometry;
@@ -262,10 +275,7 @@ gdk_wayland_screen_get_monitor_scale_factor (GdkScreen *screen,
   GdkWaylandScreen *screen_wayland = GDK_WAYLAND_SCREEN (screen);
   GdkWaylandMonitor *monitor;
 
-  if (monitor_num >= screen_wayland->monitors->len)
-    return 1;
-
-  monitor = g_ptr_array_index(screen_wayland->monitors, monitor_num);
+  monitor = g_ptr_array_index (screen_wayland->monitors, monitor_num);
 
   return monitor->scale;
 }
@@ -353,17 +363,17 @@ typedef enum
 static gdouble
 get_dpi_from_gsettings (GdkWaylandScreen *screen_wayland)
 {
-        GSettings  *interface_settings;
-        gdouble      factor;
+  GSettings *settings;
+  gdouble factor;
 
-        interface_settings = g_hash_table_lookup (screen_wayland->settings,
-                                                  "org.gnome.desktop.interface");
-        if (interface_settings != NULL)
-          factor = g_settings_get_double (interface_settings, "text-scaling-factor");
-        else
-          factor = 1.0;
+  settings = g_hash_table_lookup (screen_wayland->settings,
+                                  "org.gnome.desktop.interface");
+  if (settings != NULL)
+    factor = g_settings_get_double (settings, "text-scaling-factor");
+  else
+    factor = 1.0;
 
-        return 96.0 * factor;
+  return 96.0 * factor;
 }
 
 static void
@@ -475,11 +485,29 @@ update_xft_settings (GdkScreen *screen)
 
   if (screen_wayland->xft_settings.dpi != xft_settings.dpi)
     {
+      double dpi = xft_settings.dpi / 1024.;
+      const char *scale_env;
+      double scale;
+
       screen_wayland->xft_settings.dpi = xft_settings.dpi;
+
+      scale_env = g_getenv ("GDK_DPI_SCALE");
+      if (scale_env)
+        {
+          scale = g_ascii_strtod (scale_env, NULL);
+          if (scale != 0 && dpi > 0)
+            dpi *= scale;
+        }
+
+      _gdk_screen_set_resolution (screen, dpi);
+
       notify_setting (screen, "gtk-xft-dpi");
     }
 }
 
+#define WM_SETTINGS_SCHEMA "org.gnome.desktop.wm.preferences"
+#define CLASSIC_WM_SETTINGS_SCHEMA "org.gnome.shell.extensions.classic-overrides"
+
 typedef struct _TranslationEntry TranslationEntry;
 struct _TranslationEntry {
   const gchar *schema;
@@ -511,16 +539,21 @@ static TranslationEntry translations[] = {
   { "org.gnome.desktop.sound", "input-feedback-sounds", "gtk-enable-input-feedback-sounds", G_TYPE_BOOLEAN, { . b = FALSE } },
   { "org.gnome.desktop.privacy", "recent-files-max-age", "gtk-recent-files-max-age", G_TYPE_INT, { .i = 30 } },
   { "org.gnome.desktop.privacy", "remember-recent-files",    "gtk-recent-files-enabled", G_TYPE_BOOLEAN, { .b = TRUE } },
+  { WM_SETTINGS_SCHEMA, "button-layout",    "gtk-decoration-layout", G_TYPE_STRING, { .s = "menu:close" } },
+  { CLASSIC_WM_SETTINGS_SCHEMA, "button-layout",   "gtk-decoration-layout", G_TYPE_STRING, { .s = "menu:close" } },
   { "org.gnome.settings-daemon.plugins.xsettings", "antialiasing", "gtk-xft-antialias", G_TYPE_NONE, { .i = 0 } },
   { "org.gnome.settings-daemon.plugins.xsettings", "hinting", "gtk-xft-hinting", G_TYPE_NONE, { .i = 0 } },
   { "org.gnome.settings-daemon.plugins.xsettings", "hinting", "gtk-xft-hintstyle", G_TYPE_NONE, { .i = 0 } },
   { "org.gnome.settings-daemon.plugins.xsettings", "rgba-order", "gtk-xft-rgba", G_TYPE_NONE, { .i = 0 } },
   { "org.gnome.desktop.interface", "text-scaling-factor", "gtk-xft-dpi" , G_TYPE_NONE, { .i = 0 } },
-
+  { "org.gnome.desktop.wm.preferences", "action-double-click-titlebar", "gtk-titlebar-double-click", G_TYPE_STRING, { .s = "toggle-maximize" } },
+  { "org.gnome.desktop.wm.preferences", "action-middle-click-titlebar", "gtk-titlebar-middle-click", G_TYPE_STRING, { .s = "none" } },
+  { "org.gnome.desktop.wm.preferences", "action-right-click-titlebar", "gtk-titlebar-right-click", G_TYPE_STRING, { .s = "menu" } },
 };
 
 static TranslationEntry *
-find_translation_entry_by_key (GSettings *settings, const gchar *key)
+find_translation_entry_by_key (GSettings   *settings,
+                               const gchar *key)
 {
   guint i;
   gchar *schema;
@@ -611,9 +644,14 @@ gtk_shell_handle_capabilities (void             *data,
 			       struct gtk_shell *shell,
 			       uint32_t          capabilities)
 {
-  GdkWaylandScreen *screen_wayland = data;
+  GdkScreen *screen = data;
+  GdkWaylandScreen *screen_wayland = GDK_WAYLAND_SCREEN (data);
 
   screen_wayland->shell_capabilities = capabilities;
+
+  notify_setting (screen, "gtk-shell-shows-app-menu");
+  notify_setting (screen, "gtk-shell-shows-menubar");
+  notify_setting (screen, "gtk-shell-shows-desktop");
 }
 
 struct gtk_shell_listener gdk_screen_gtk_shell_listener = {
@@ -681,10 +719,45 @@ set_value_from_entry (GdkScreen        *screen,
     }
 }
 
+static void
+set_decoration_layout_from_entry (GdkScreen        *screen,
+                                  TranslationEntry *entry,
+                                  GValue           *value)
+{
+  GdkWaylandScreen *screen_wayland = GDK_WAYLAND_SCREEN (screen);
+  GSettings *settings = NULL;
+  const char *session;
+
+  /* Hack: until we get session-dependent defaults in GSettings,
+   *       swap out the usual schema for the "classic" one when
+   *       running in classic mode
+   */
+  session = g_getenv ("XDG_CURRENT_DESKTOP");
+  if (session && strstr (session, "GNOME-Classic"))
+    settings = (GSettings *)g_hash_table_lookup (screen_wayland->settings, CLASSIC_WM_SETTINGS_SCHEMA);
+
+  if (settings == NULL)
+    settings = (GSettings *)g_hash_table_lookup (screen_wayland->settings, WM_SETTINGS_SCHEMA);
+
+  if (settings)
+    {
+      gchar *s = g_settings_get_string (settings, entry->key);
+
+      translate_wm_button_layout_to_gtk (s);
+      g_value_set_string (value, s);
+
+      g_free (s);
+    }
+  else
+    {
+      g_value_set_static_string (value, entry->fallback.s);
+    }
+}
+
 static gboolean
 set_capability_setting (GdkScreen                 *screen,
-			GValue                    *value,
-			enum gtk_shell_capability  test)
+                        GValue                    *value,
+                        enum gtk_shell_capability  test)
 {
   GdkWaylandScreen *wayland_screen = GDK_WAYLAND_SCREEN (screen);
 
@@ -695,8 +768,8 @@ set_capability_setting (GdkScreen                 *screen,
 
 static gboolean
 gdk_wayland_screen_get_setting (GdkScreen   *screen,
-				const gchar *name,
-				GValue      *value)
+                                const gchar *name,
+                                GValue      *value)
 {
   TranslationEntry *entry;
 
@@ -705,7 +778,10 @@ gdk_wayland_screen_get_setting (GdkScreen   *screen,
   entry = find_translation_entry_by_setting (name);
   if (entry != NULL)
     {
-      set_value_from_entry (screen, entry, value);
+      if (strcmp (name, "gtk-decoration-layout") == 0)
+        set_decoration_layout_from_entry (screen, entry, value);
+      else
+        set_value_from_entry (screen, entry, value);
       return TRUE;
    }
 
@@ -718,6 +794,12 @@ gdk_wayland_screen_get_setting (GdkScreen   *screen,
   if (strcmp (name, "gtk-shell-shows-desktop") == 0)
     return set_capability_setting (screen, value, GTK_SHELL_CAPABILITY_DESKTOP_ICONS);
 
+  if (strcmp (name, "gtk-dialogs-use-header") == 0)
+    {
+      g_value_set_boolean (value, TRUE);
+      return TRUE;
+    }
+
   return FALSE;
 }
 
@@ -734,6 +816,8 @@ struct _GdkWaylandVisualClass
   GdkVisualClass parent_class;
 };
 
+GType _gdk_wayland_visual_get_type (void);
+
 G_DEFINE_TYPE (GdkWaylandVisual, _gdk_wayland_visual, GDK_TYPE_VISUAL)
 
 static void
@@ -834,6 +918,7 @@ gdk_wayland_visual_new (GdkScreen *screen)
   visual->screen = GDK_SCREEN (screen);
   visual->type = GDK_VISUAL_TRUE_COLOR;
   visual->depth = 32;
+  visual->bits_per_rgb = 8;
 
   return visual;
 }
@@ -916,33 +1001,65 @@ _gdk_wayland_screen_init (GdkWaylandScreen *screen_wayland)
 static void
 update_screen_size (GdkWaylandScreen *screen_wayland)
 {
+  gboolean emit_changed = FALSE;
   gint width, height;
+  gint width_mm, height_mm;
   gint i;
 
   width = height = 0;
+  width_mm = height_mm = 0;
   for (i = 0; i < screen_wayland->monitors->len; i++)
     {
       GdkWaylandMonitor *monitor = screen_wayland->monitors->pdata[i];
 
+      /* XXX: Largely assuming here that monitor areas
+       * are contiguous and never overlap.
+       */
+      if (monitor->geometry.x > 0)
+        width_mm += monitor->width_mm;
+      else
+        width_mm = MAX (width_mm, monitor->width_mm);
+
+      if (monitor->geometry.y > 0)
+        height_mm += monitor->height_mm;
+      else
+        height_mm = MAX (height_mm, monitor->height_mm);
+
       width = MAX (width, monitor->geometry.x + monitor->geometry.width);
       height = MAX (height, monitor->geometry.y + monitor->geometry.height);
     }
 
+  if (screen_wayland->width_mm != width_mm ||
+      screen_wayland->height_mm != height_mm)
+    {
+      emit_changed = TRUE;
+      screen_wayland->width_mm = width_mm;
+      screen_wayland->height_mm = height_mm;
+    }
+
   if (screen_wayland->width != width ||
       screen_wayland->height != height)
     {
+      emit_changed = TRUE;
       screen_wayland->width = width;
       screen_wayland->height = height;
-      g_signal_emit_by_name (screen_wayland, "size-changed");
     }
+
+  if (emit_changed)
+    g_signal_emit_by_name (screen_wayland, "size-changed");
 }
 
 static void
-output_handle_geometry(void *data,
-		       struct wl_output *wl_output,
-		       int x, int y, int physical_width, int physical_height,
-		       int subpixel, const char *make, const char *model,
-		       int32_t transform)
+output_handle_geometry (void             *data,
+                        struct wl_output *wl_output,
+                        int               x,
+                        int               y,
+                        int               physical_width,
+                        int               physical_height,
+                        int               subpixel,
+                        const char       *make,
+                        const char       *model,
+                        int32_t           transform)
 {
   GdkWaylandMonitor *monitor = (GdkWaylandMonitor *)data;
 
@@ -963,8 +1080,8 @@ output_handle_geometry(void *data,
 }
 
 static void
-output_handle_done(void *data,
-		   struct wl_output *wl_output)
+output_handle_done (void             *data,
+                    struct wl_output *wl_output)
 {
   GdkWaylandMonitor *monitor = (GdkWaylandMonitor *)data;
 
@@ -973,22 +1090,23 @@ output_handle_done(void *data,
 }
 
 static void
-output_handle_scale(void *data,
-		    struct wl_output *wl_output,
-		    int32_t factor)
+output_handle_scale (void             *data,
+                     struct wl_output *wl_output,
+                     int32_t           factor)
 {
   GdkWaylandMonitor *monitor = (GdkWaylandMonitor *)data;
 
   monitor->scale = factor;
+  g_signal_emit_by_name (monitor->screen, "monitors-changed");
 }
 
 static void
-output_handle_mode(void *data,
-                   struct wl_output *wl_output,
-                   uint32_t flags,
-                   int width,
-                   int height,
-                   int refresh)
+output_handle_mode (void             *data,
+                    struct wl_output *wl_output,
+                    uint32_t          flags,
+                    int               width,
+                    int               height,
+                    int               refresh)
 {
   GdkWaylandMonitor *monitor = (GdkWaylandMonitor *)data;
 
@@ -1021,16 +1139,19 @@ _gdk_wayland_screen_add_output (GdkScreen        *screen,
 				guint32           version)
 {
   GdkWaylandScreen *screen_wayland = GDK_WAYLAND_SCREEN (screen);
-  GdkWaylandMonitor *monitor = g_new0(GdkWaylandMonitor, 1);
+  GdkWaylandMonitor *monitor;
+
+  monitor = g_new0 (GdkWaylandMonitor, 1);
 
   monitor->id = id;
   monitor->output = output;
   monitor->version = version;
   monitor->screen = screen_wayland;
   monitor->scale = 1;
-  g_ptr_array_add(screen_wayland->monitors, monitor);
 
-  wl_output_add_listener(output, &output_listener, monitor);
+  g_ptr_array_add (screen_wayland->monitors, monitor);
+
+  wl_output_add_listener (output, &output_listener, monitor);
 }
 
 void
