@@ -49,8 +49,65 @@
  *
  * A GtkImageMenuItem is a menu item which has an icon next to the text label.
  *
- * Note that the user can disable display of menu icons, so make sure to still
- * fill in the text label.
+ * This is functionally equivalent to:
+ *
+ * |[<!-- language="C" -->
+ *   GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+ *   GtkWidget *icon = gtk_image_new_from_icon_name ("folder-music-symbolic", GTK_ICON_SIZE_MENU);
+ *   GtkWidget *label = gtk_label_new ("Music");
+ *   GtkWidget *menu_item = gtk_menu_item_new ();
+ *
+ *   gtk_container_add (GTK_CONTAINER (box), icon);
+ *   gtk_container_add (GTK_CONTAINER (box), label);
+ *
+ *   gtk_container_add (GTK_CONTAINER (menu_item), box);
+ *
+ *   gtk_widget_show_all (menu_item);
+ * ]|
+ *
+ * Note that the user may disable display of menu icons using
+ * the #GtkSettings:gtk-menu-images setting, so make sure to still
+ * fill in the text label. If you want to ensure that your menu items
+ * show an icon you are strongly encouraged to use a #GtkMenuItem
+ * with a #GtkImage instead.
+ *
+ * #GtkImageMenuItem has been deprecated since GTK+ 3.10. If you want to
+ * display an icon in a menu item, you should use #GtkMenuItem and pack a
+ * #GtkBox with a #GtkImage and a #GtkLabel instead. You should also consider
+ * using #GtkBuilder and the XML #GMenu description for creating menus, by
+ * following the [GMenu guide][https://developer.gnome.org/GMenu/]. You should
+ * consider using icons in menu items only sparingly, and for "objects" (or
+ * "nouns") elements only, like bookmarks, files, and links; "actions" (or
+ * "verbs") should not have icons.
+ *
+ * Furthermore, if you would like to display keyboard accelerator, you must
+ * pack the accel label into the box using gtk_box_pack_end() and align the
+ * label, otherwise the accelerator will not display correctly. The following
+ * code snippet adds a keyboard accelerator to the menu item, with a key
+ * binding of Ctrl+M:
+ *
+ * |[<!-- language="C" -->
+ *   GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+ *   GtkWidget *icon = gtk_image_new_from_icon_name ("folder-music-symbolic", GTK_ICON_SIZE_MENU);
+ *   GtkWidget *label = gtk_accel_label_new ("Music");
+ *   GtkWidget *menu_item = gtk_menu_item_new ();
+ *   GtkAccelGroup *accel_group = gtk_accel_group_new ();
+ *
+ *   gtk_container_add (GTK_CONTAINER (box), icon);
+ *
+ *   gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
+ *   gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+ *
+ *   gtk_widget_add_accelerator (menu_item, "activate", accel_group,
+ *                               GDK_KEY_m, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
+ *   gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (label), menu_item);
+ *
+ *   gtk_box_pack_end (GTK_BOX (box), label, TRUE, TRUE, 0);
+ *
+ *   gtk_container_add (GTK_CONTAINER (menu_item), box);
+ *
+ *   gtk_widget_show_all (menu_item);
+ * ]|
  */
 
 
@@ -160,7 +217,8 @@ gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
    *
    * Child widget to appear next to the menu text.
    *
-   * Deprecated: 3.10
+   * Deprecated: 3.10: Use a #GtkMenuItem containing a #GtkBox with
+   *   a #GtkAccelLabel and a #GtkImage instead
    */
   g_object_class_install_property (gobject_class,
                                    PROP_IMAGE,
@@ -177,7 +235,7 @@ gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
    *
    * Since: 2.16
    *
-   * Deprecated: 3.10
+   * Deprecated: 3.10: Use a named icon from the #GtkIconTheme instead
    */
   g_object_class_install_property (gobject_class,
                                    PROP_USE_STOCK,
@@ -197,7 +255,8 @@ gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
    *
    * Since: 2.16
    *
-   * Deprecated: 3.10
+   * Deprecated: 3.10: Use a #GtkMenuItem containing a #GtkBox with
+   *   a #GtkAccelLabel and a #GtkImage instead
    */
   g_object_class_install_property (gobject_class,
                                    PROP_ALWAYS_SHOW_IMAGE,
@@ -214,7 +273,7 @@ gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
    *
    * Since: 2.16
    *
-   * Deprecated: 3.10
+   * Deprecated: 3.10: Use gtk_widget_add_accelerator() instead 
    */
   g_object_class_install_property (gobject_class,
                                    PROP_ACCEL_GROUP,
@@ -672,22 +731,35 @@ gtk_image_menu_item_activatable_interface_init (GtkActivatableIface  *iface)
   iface->sync_action_properties = gtk_image_menu_item_sync_action_properties;
 }
 
+static GtkWidget *
+gtk_image_menu_item_ensure_image (GtkImageMenuItem *item)
+{
+  GtkWidget *image;
+
+  image = gtk_image_menu_item_get_image (item);
+  if (!GTK_IS_IMAGE (image))
+    {
+      image = gtk_image_new ();
+      gtk_widget_show (image);
+      gtk_image_menu_item_set_image (item, image);
+    }
+
+  return image;
+}
+
 static gboolean
 activatable_update_stock_id (GtkImageMenuItem *image_menu_item, GtkAction *action)
 {
-  GtkWidget   *image;
   const gchar *stock_id  = gtk_action_get_stock_id (action);
 
-  image = gtk_image_menu_item_get_image (image_menu_item);
-
   G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 
-  if (GTK_IS_IMAGE (image) &&
-      stock_id && gtk_icon_factory_lookup_default (stock_id))
+  if (stock_id && gtk_icon_factory_lookup_default (stock_id))
     {
-      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
+      GtkWidget *image;
+
+      image = gtk_image_menu_item_ensure_image (image_menu_item);
       gtk_image_set_from_stock (GTK_IMAGE (image), stock_id, GTK_ICON_SIZE_MENU);
-      G_GNUC_END_IGNORE_DEPRECATIONS;
       return TRUE;
     }
 
@@ -699,7 +771,6 @@ activatable_update_stock_id (GtkImageMenuItem *image_menu_item, GtkAction *actio
 static gboolean
 activatable_update_gicon (GtkImageMenuItem *image_menu_item, GtkAction *action)
 {
-  GtkWidget   *image;
   GIcon       *icon = gtk_action_get_gicon (action);
   const gchar *stock_id;
   gboolean     ret = FALSE;
@@ -708,11 +779,11 @@ activatable_update_gicon (GtkImageMenuItem *image_menu_item, GtkAction *action)
 
   G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 
-  image = gtk_image_menu_item_get_image (image_menu_item);
-
-  if (icon && GTK_IS_IMAGE (image) &&
-      !(stock_id && gtk_icon_factory_lookup_default (stock_id)))
+  if (icon && !(stock_id && gtk_icon_factory_lookup_default (stock_id)))
     {
+      GtkWidget *image;
+
+      image = gtk_image_menu_item_ensure_image (image_menu_item);
       gtk_image_set_from_gicon (GTK_IMAGE (image), icon, GTK_ICON_SIZE_MENU);
       ret = TRUE;
     }
@@ -722,20 +793,21 @@ activatable_update_gicon (GtkImageMenuItem *image_menu_item, GtkAction *action)
   return ret;
 }
 
-static void
+static gboolean
 activatable_update_icon_name (GtkImageMenuItem *image_menu_item, GtkAction *action)
 {
-  GtkWidget   *image;
   const gchar *icon_name = gtk_action_get_icon_name (action);
 
-  image = gtk_image_menu_item_get_image (image_menu_item);
-
-  if (GTK_IS_IMAGE (image) &&
-      (gtk_image_get_storage_type (GTK_IMAGE (image)) == GTK_IMAGE_EMPTY ||
-       gtk_image_get_storage_type (GTK_IMAGE (image)) == GTK_IMAGE_ICON_NAME))
+  if (icon_name)
     {
+      GtkWidget *image;
+
+      image = gtk_image_menu_item_ensure_image (image_menu_item);
       gtk_image_set_from_icon_name (GTK_IMAGE (image), icon_name, GTK_ICON_SIZE_MENU);
+      return TRUE;
     }
+
+  return FALSE;
 }
 
 static void
@@ -767,7 +839,6 @@ gtk_image_menu_item_sync_action_properties (GtkActivatable *activatable,
                                             GtkAction      *action)
 {
   GtkImageMenuItem *image_menu_item;
-  GtkWidget *image;
   gboolean   use_appearance;
 
   image_menu_item = GTK_IMAGE_MENU_ITEM (activatable);
@@ -781,21 +852,6 @@ gtk_image_menu_item_sync_action_properties (GtkActivatable *activatable,
   if (!use_appearance)
     return;
 
-  image = gtk_image_menu_item_get_image (image_menu_item);
-  if (image && !GTK_IS_IMAGE (image))
-    {
-      gtk_image_menu_item_set_image (image_menu_item, NULL);
-      image = NULL;
-    }
-
-  if (!image)
-    {
-      image = gtk_image_new ();
-      gtk_widget_show (image);
-      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (activatable),
-                                     image);
-    }
-
   if (!activatable_update_stock_id (image_menu_item, action) &&
       !activatable_update_gicon (image_menu_item, action))
     activatable_update_icon_name (image_menu_item, action);
@@ -878,7 +934,7 @@ gtk_image_menu_item_new_with_mnemonic (const gchar *label)
  *
  * Returns: a new #GtkImageMenuItem.
  *
- * Deprecated: 3.10: Use gtk_menu_item_new() instead.
+ * Deprecated: 3.10: Use gtk_menu_item_new_with_mnemonic() instead.
  */
 GtkWidget*
 gtk_image_menu_item_new_from_stock (const gchar   *stock_id,
@@ -1098,6 +1154,7 @@ gtk_image_menu_item_set_image (GtkImageMenuItem *image_menu_item,
                 "visible", show_image (image_menu_item),
                 "no-show-all", TRUE,
                 NULL);
+  gtk_image_set_pixel_size (GTK_IMAGE (image), 16);
 
   g_object_notify (G_OBJECT (image_menu_item), "image");
 }
@@ -1109,7 +1166,7 @@ gtk_image_menu_item_set_image (GtkImageMenuItem *image_menu_item,
  * Gets the widget that is currently set as the image of @image_menu_item.
  * See gtk_image_menu_item_set_image().
  *
- * Return value: (transfer none): the widget set as image of @image_menu_item
+ * Returns: (transfer none): the widget set as image of @image_menu_item
  *
  * Deprecated: 3.10
  **/
