@@ -21,6 +21,8 @@
 
 #include "gtkcssshadowsvalueprivate.h"
 
+#include <math.h>
+
 #include "gtkcairoblurprivate.h"
 #include "gtkcssshadowvalueprivate.h"
 
@@ -52,9 +54,8 @@ static GtkCssValue *
 gtk_css_value_shadows_compute (GtkCssValue             *value,
                                guint                    property_id,
                                GtkStyleProviderPrivate *provider,
-			       int                      scale,
-                               GtkCssComputedValues    *values,
-                               GtkCssComputedValues    *parent_values,
+                               GtkCssStyle             *style,
+                               GtkCssStyle             *parent_style,
                                GtkCssDependencies      *dependencies)
 {
   GtkCssValue *result;
@@ -67,7 +68,7 @@ gtk_css_value_shadows_compute (GtkCssValue             *value,
   result = gtk_css_shadows_value_new (value->values, value->len);
   for (i = 0; i < value->len; i++)
     {
-      result->values[i] = _gtk_css_value_compute (value->values[i], property_id, provider, scale, values, parent_values, &child_deps);
+      result->values[i] = _gtk_css_value_compute (value->values[i], property_id, provider, style, parent_style, &child_deps);
       *dependencies = _gtk_css_dependencies_union (*dependencies, child_deps);
     }
 
@@ -213,7 +214,8 @@ gtk_css_shadows_value_new (GtkCssValue **values,
 }
 
 GtkCssValue *
-_gtk_css_shadows_value_parse (GtkCssParser *parser)
+_gtk_css_shadows_value_parse (GtkCssParser *parser,
+                              gboolean      box_shadow_mode)
 {
   GtkCssValue *value, *result;
   GPtrArray *values;
@@ -224,7 +226,7 @@ _gtk_css_shadows_value_parse (GtkCssParser *parser)
   values = g_ptr_array_new ();
 
   do {
-    value = _gtk_css_shadow_value_parse (parser);
+    value = _gtk_css_shadow_value_parse (parser, box_shadow_mode);
 
     if (value == NULL)
       {
@@ -241,6 +243,14 @@ _gtk_css_shadows_value_parse (GtkCssParser *parser)
   return result;
 }
 
+gboolean
+_gtk_css_shadows_value_is_none (const GtkCssValue *shadows)
+{
+  g_return_val_if_fail (shadows->class == &GTK_CSS_VALUE_SHADOWS, TRUE);
+
+  return shadows->len == 0;
+}
+
 void
 _gtk_css_shadows_value_paint_layout (const GtkCssValue *shadows,
                                      cairo_t           *cr,
@@ -270,22 +280,6 @@ _gtk_css_shadows_value_paint_icon (const GtkCssValue *shadows,
     }
 }
 
-void
-_gtk_css_shadows_value_paint_spinner (const GtkCssValue *shadows,
-                                      cairo_t           *cr,
-                                      gdouble            radius,
-                                      gdouble            progress)
-{
-  guint i;
-
-  g_return_if_fail (shadows->class == &GTK_CSS_VALUE_SHADOWS);
-
-  for (i = 0; i < shadows->len; i++)
-    {
-      _gtk_css_shadow_value_paint_spinner (shadows->values[i], cr, radius, progress);
-    }
-}
-
 void
 _gtk_css_shadows_value_paint_box (const GtkCssValue   *shadows,
                                   cairo_t             *cr,
@@ -314,6 +308,8 @@ _gtk_css_shadows_value_get_extents (const GtkCssValue *shadows,
 
   g_return_if_fail (shadows->class == &GTK_CSS_VALUE_SHADOWS);
 
+  *border = b;
+
   for (i = 0; i < shadows->len; i++)
     {
       shadow = shadows->values[i];
@@ -326,10 +322,10 @@ _gtk_css_shadows_value_get_extents (const GtkCssValue *shadows,
                                           &radius, &spread);
       clip_radius = _gtk_cairo_blur_compute_pixels (radius);
 
-      b.top = MAX (0, clip_radius + spread - voffset);
-      b.right = MAX (0, clip_radius + spread + hoffset);
-      b.bottom = MAX (0, clip_radius + spread + voffset);
-      b.left = MAX (0, clip_radius + spread - hoffset);
+      b.top = MAX (0, ceil (clip_radius + spread - voffset));
+      b.right = MAX (0, ceil (clip_radius + spread + hoffset));
+      b.bottom = MAX (0, ceil (clip_radius + spread + voffset));
+      b.left = MAX (0, ceil (clip_radius + spread - hoffset));
 
       border->top = MAX (border->top, b.top);
       border->right = MAX (border->right, b.right);
