@@ -43,9 +43,8 @@ struct _GtkCssValueClass {
   GtkCssValue * (* compute)                           (GtkCssValue                *value,
                                                        guint                       property_id,
                                                        GtkStyleProviderPrivate    *provider,
-						       int                         scale,
-                                                       GtkCssComputedValues       *values,
-                                                       GtkCssComputedValues       *parent_values,
+                                                       GtkCssStyle                *style,
+                                                       GtkCssStyle                *parent_style,
                                                        GtkCssDependencies         *dependencies);
   gboolean      (* equal)                             (const GtkCssValue          *value1,
                                                        const GtkCssValue          *value2);
@@ -69,9 +68,8 @@ void         _gtk_css_value_unref                     (GtkCssValue
 GtkCssValue *_gtk_css_value_compute                   (GtkCssValue                *value,
                                                        guint                       property_id,
                                                        GtkStyleProviderPrivate    *provider,
-						       int                         scale,
-                                                       GtkCssComputedValues       *values,
-                                                       GtkCssComputedValues       *parent_values,
+                                                       GtkCssStyle                *style,
+                                                       GtkCssStyle                *parent_style,
                                                        GtkCssDependencies         *dependencies);
 gboolean     _gtk_css_value_equal                     (const GtkCssValue          *value1,
                                                        const GtkCssValue          *value2);
