@@ -46,6 +46,7 @@
 #include "gtktreeselection.h"
 #include "gtktreeview.h"
 #include "gtkwidget.h"
+#include "gtksettings.h"
 
 /**
  * SECTION:gtkfontchooserwidget
@@ -95,6 +96,8 @@ struct _GtkFontChooserWidgetPrivate
   GtkFontFilterFunc filter_func;
   gpointer          filter_data;
   GDestroyNotify    filter_data_destroy;
+
+  guint last_fontconfig_timestamp;
 };
 
 /* This is the initial fixed height and the top padding of the preview entry */
@@ -262,6 +265,23 @@ size_change_cb (GtkAdjustment *adjustment,
   gtk_font_chooser_widget_take_font_desc (fontchooser, font_desc);
 }
 
+static gboolean
+output_cb (GtkSpinButton *spin,
+           gpointer       data)
+{
+  GtkAdjustment *adjustment;
+  gchar *text;
+  gdouble value;
+
+  adjustment = gtk_spin_button_get_adjustment (spin);
+  value = gtk_adjustment_get_value (adjustment);
+  text = g_strdup_printf ("%2.4g", value);
+  gtk_entry_set_text (GTK_ENTRY (spin), text);
+  g_free (text);
+
+  return TRUE;
+}
+
 static void
 gtk_font_chooser_widget_update_marks (GtkFontChooserWidget *fontchooser)
 {
@@ -360,34 +380,24 @@ static PangoFontDescription *
 tree_model_get_font_description (GtkTreeModel *model,
                                  GtkTreeIter  *iter)
 {
-  PangoFontDescription *desc;
+  PangoFontDescription *desc, *face_desc;
   PangoFontFace *face;
-  GtkTreeIter child_iter;
 
   gtk_tree_model_get (model, iter,
                       FONT_DESC_COLUMN, &desc,
                       -1);
-  if (desc != NULL)
+  if (pango_font_description_get_set_fields (desc) != 0)
     return desc;
 
   gtk_tree_model_get (model, iter,
                       FACE_COLUMN, &face,
                       -1);
-  desc = pango_font_face_describe (face);
+  face_desc = pango_font_face_describe (face);
   g_object_unref (face);
   
-  if (GTK_IS_TREE_MODEL_FILTER (model))
-    {
-      gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model),
-                                                        &child_iter,
-                                                        iter);
-      iter = &child_iter;
-      model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
-    }
+  pango_font_description_merge (desc, face_desc, TRUE);
 
-  gtk_list_store_set (GTK_LIST_STORE (model), iter,
-                      FONT_DESC_COLUMN, desc,
-                      -1);
+  pango_font_description_free (face_desc);
 
   return desc;
 }
@@ -452,6 +462,26 @@ resize_by_scroll_cb (GtkWidget      *scrolled_window,
   return TRUE;
 }
 
+static void
+gtk_font_chooser_widget_update_preview_attributes (GtkFontChooserWidget *fontchooser)
+{
+  GtkFontChooserWidgetPrivate *priv;
+  PangoAttrList *attrs;
+
+  priv = fontchooser->priv;
+
+  attrs = pango_attr_list_new ();
+
+  /* Prevent font fallback */
+  pango_attr_list_insert (attrs, pango_attr_fallback_new (FALSE));
+
+  /* Force current font */
+  pango_attr_list_insert (attrs, pango_attr_font_desc_new (priv->font_desc));
+
+  gtk_entry_set_attributes (GTK_ENTRY (priv->preview), attrs);
+  pango_attr_list_unref (attrs);
+}
+
 static void
 row_inserted_cb (GtkTreeModel *model,
                  GtkTreePath  *path,
@@ -493,7 +523,7 @@ gtk_font_chooser_widget_class_init (GtkFontChooserWidgetClass *klass)
 
   /* Bind class to template */
   gtk_widget_class_set_template_from_resource (widget_class,
-					       "/org/gtk/libgtk/gtkfontchooserwidget.ui");
+					       "/org/gtk/libgtk/ui/gtkfontchooserwidget.ui");
 
   gtk_widget_class_bind_template_child_private (widget_class, GtkFontChooserWidget, search_entry);
   gtk_widget_class_bind_template_child_private (widget_class, GtkFontChooserWidget, family_face_list);
@@ -516,6 +546,7 @@ gtk_font_chooser_widget_class_init (GtkFontChooserWidgetClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, row_inserted_cb);
   gtk_widget_class_bind_template_callback (widget_class, row_deleted_cb);
   gtk_widget_class_bind_template_callback (widget_class, size_change_cb);
+  gtk_widget_class_bind_template_callback (widget_class, output_cb);
 }
 
 static void
@@ -534,8 +565,10 @@ gtk_font_chooser_widget_init (GtkFontChooserWidget *fontchooser)
   priv->font_desc = pango_font_description_new ();
 
   /* Set default preview text */
-  gtk_entry_set_text (GTK_ENTRY (priv->preview),
-                      pango_language_get_sample_string (NULL));
+  gtk_entry_set_text (GTK_ENTRY (priv->preview), priv->preview_text);
+
+  gtk_font_chooser_widget_update_preview_attributes (fontchooser);
+
   gtk_widget_add_events (priv->preview, GDK_SCROLL_MASK);
 
   /* Set the upper values of the spin/scale with G_MAXINT / PANGO_SCALE */
@@ -554,7 +587,7 @@ gtk_font_chooser_widget_init (GtkFontChooserWidget *fontchooser)
                                            fontchooser,
                                            NULL);
 
-  /* Load data and set initial style dependant parameters */
+  /* Load data and set initial style-dependent parameters */
   gtk_font_chooser_widget_load_fonts (fontchooser);
   gtk_font_chooser_widget_set_cell_size (fontchooser);
   gtk_font_chooser_widget_take_font_desc (fontchooser, NULL);
@@ -565,7 +598,7 @@ gtk_font_chooser_widget_init (GtkFontChooserWidget *fontchooser)
  *
  * Creates a new #GtkFontChooserWidget.
  *
- * Return value: a new #GtkFontChooserWidget
+ * Returns: a new #GtkFontChooserWidget
  *
  * Since: 3.2
  */
@@ -593,6 +626,21 @@ gtk_font_chooser_widget_load_fonts (GtkFontChooserWidget *fontchooser)
   gint n_families, i;
   PangoFontFamily **families;
   gchar *family_and_face;
+  guint fontconfig_timestamp;
+
+  g_object_get (gtk_widget_get_settings (GTK_WIDGET (fontchooser)),
+                "gtk-fontconfig-timestamp", &fontconfig_timestamp,
+                NULL);
+
+  /* The fontconfig timestamp is only set on systems with fontconfig; every
+   * other platform will set it to 0. For those systems, we fall back to
+   * reloading the fonts every time.
+   */
+  if (fontconfig_timestamp != 0 &&
+      priv->last_fontconfig_timestamp == fontconfig_timestamp)
+    return;
+
+  priv->last_fontconfig_timestamp = fontconfig_timestamp;
 
   list_store = GTK_LIST_STORE (priv->model);
 
@@ -602,7 +650,9 @@ gtk_font_chooser_widget_load_fonts (GtkFontChooserWidget *fontchooser)
 
   qsort (families, n_families, sizeof (PangoFontFamily *), cmp_families);
 
+  g_signal_handlers_block_by_func(priv->family_face_list, cursor_changed_cb, fontchooser);
   gtk_list_store_clear (list_store);
+  g_signal_handlers_unblock_by_func (priv->family_face_list, cursor_changed_cb, fontchooser);
 
   /* Iterate over families and faces */
   for (i = 0; i < n_families; i++)
@@ -616,19 +666,23 @@ gtk_font_chooser_widget_load_fonts (GtkFontChooserWidget *fontchooser)
 
       for (j = 0; j < n_faces; j++)
         {
+          PangoFontDescription *empty_font_desc;
           const gchar *face_name;
 
           face_name = pango_font_face_get_face_name (faces[j]);
 
           family_and_face = g_strconcat (fam_name, " ", face_name, NULL);
+          empty_font_desc = pango_font_description_new ();
 
           gtk_list_store_insert_with_values (list_store, &iter, -1,
                                              FAMILY_COLUMN, families[i],
                                              FACE_COLUMN, faces[j],
+                                             FONT_DESC_COLUMN, empty_font_desc,
                                              PREVIEW_TITLE_COLUMN, family_and_face,
                                              -1);
 
           g_free (family_and_face);
+          pango_font_description_free (empty_font_desc);
         }
 
       g_free (faces);
@@ -770,16 +824,18 @@ gtk_font_chooser_widget_cell_data_func (GtkTreeViewColumn *column,
   GtkFontChooserWidget *fontchooser = user_data;
   PangoFontDescription *font_desc;
   PangoAttrList *attrs;
-  char *to_string, *text;
+  char *preview_title, *text;
   gsize first_line_len;
 
   font_desc = tree_model_get_font_description (tree_model, iter);
 
-  to_string = pango_font_description_to_string (font_desc);
+  gtk_tree_model_get (tree_model, iter,
+                      PREVIEW_TITLE_COLUMN, &preview_title,
+                      -1);
+
+  text = g_strconcat (preview_title, "\n", fontchooser->priv->preview_text, NULL);
+  first_line_len = strlen (preview_title) + 1;
 
-  text = g_strconcat (to_string, "\n", fontchooser->priv->preview_text, NULL);
-  first_line_len = strlen (to_string) + 1;
-  
   attrs = gtk_font_chooser_widget_get_preview_attributes (fontchooser, 
                                                           font_desc,
                                                           first_line_len);
@@ -791,8 +847,8 @@ gtk_font_chooser_widget_cell_data_func (GtkTreeViewColumn *column,
 
   pango_font_description_free (font_desc);
   pango_attr_list_unref (attrs);
-  g_free (to_string);
   g_free (text);
+  g_free (preview_title);
 }
 
 static void
@@ -834,6 +890,8 @@ gtk_font_chooser_widget_finalize (GObject *object)
   if (priv->filter_data_destroy)
     priv->filter_data_destroy (priv->filter_data);
 
+  g_free (priv->preview_text);
+
   G_OBJECT_CLASS (gtk_font_chooser_widget_parent_class)->finalize (object);
 }
 
@@ -851,8 +909,6 @@ gtk_font_chooser_widget_find_font (GtkFontChooserWidget        *fontchooser,
                                    GtkTreeIter                 *iter)
 {
   GtkFontChooserWidgetPrivate *priv = fontchooser->priv;
-  PangoFontDescription *desc;
-  PangoFontFamily *family;
   gboolean valid;
 
   if (pango_font_description_get_family (font_desc) == NULL)
@@ -862,6 +918,9 @@ gtk_font_chooser_widget_find_font (GtkFontChooserWidget        *fontchooser,
        valid;
        valid = gtk_tree_model_iter_next (priv->model, iter))
     {
+      PangoFontDescription *desc;
+      PangoFontFamily *family;
+
       gtk_tree_model_get (priv->model, iter,
                           FAMILY_COLUMN, &family,
                           -1);
@@ -873,8 +932,10 @@ gtk_font_chooser_widget_find_font (GtkFontChooserWidget        *fontchooser,
       desc = tree_model_get_font_description (priv->model, iter);
 
       pango_font_description_merge_static (desc, font_desc, FALSE);
-      if (pango_font_description_equal (desc, font_desc))
+      if (pango_font_description_equal (desc, font_desc)) {
+        pango_font_description_free (desc);
         break;
+      }
 
       pango_font_description_free (desc);
     }
@@ -887,10 +948,20 @@ gtk_font_chooser_widget_screen_changed (GtkWidget *widget,
                                         GdkScreen *previous_screen)
 {
   GtkFontChooserWidget *fontchooser = GTK_FONT_CHOOSER_WIDGET (widget);
+  GtkSettings *settings;
 
   if (GTK_WIDGET_CLASS (gtk_font_chooser_widget_parent_class)->screen_changed)
     GTK_WIDGET_CLASS (gtk_font_chooser_widget_parent_class)->screen_changed (widget, previous_screen);
 
+  if (previous_screen)
+    {
+      settings = gtk_settings_get_for_screen (previous_screen);
+      g_signal_handlers_disconnect_by_func (settings, gtk_font_chooser_widget_load_fonts, widget);
+    }
+  settings = gtk_widget_get_settings (widget);
+  g_signal_connect_object (settings, "notify::gtk-fontconfig-timestamp",
+                           G_CALLBACK (gtk_font_chooser_widget_load_fonts), widget, G_CONNECT_SWAPPED);
+
   if (previous_screen == NULL)
     previous_screen = gdk_screen_get_default ();
 
@@ -1051,7 +1122,7 @@ gtk_font_chooser_widget_merge_font_desc (GtkFontChooserWidget *fontchooser,
       gtk_font_chooser_widget_update_marks (fontchooser);
     }
 
-  gtk_widget_override_font (priv->preview, priv->font_desc);
+  gtk_font_chooser_widget_update_preview_attributes (fontchooser);
 
   pango_font_description_free (font_desc); /* adopted */
 
@@ -1075,26 +1146,14 @@ gtk_font_chooser_widget_take_font_desc (GtkFontChooserWidget *fontchooser,
     {
       GtkTreeIter iter;
 
-      if (gtk_font_chooser_widget_find_font (fontchooser,
-                                             font_desc,
-                                             &iter))
-        {
-          gtk_font_chooser_widget_merge_font_desc (fontchooser,
-                                                   font_desc,
-                                                   &iter);
-        }
+      if (gtk_font_chooser_widget_find_font (fontchooser, font_desc, &iter))
+        gtk_font_chooser_widget_merge_font_desc (fontchooser, font_desc, &iter);
       else
-        {
-          gtk_font_chooser_widget_merge_font_desc (fontchooser,
-                                                   font_desc,
-                                                   NULL);
-        }
+        gtk_font_chooser_widget_merge_font_desc (fontchooser, font_desc, NULL);
     }
   else
     {
-      gtk_font_chooser_widget_merge_font_desc (fontchooser,
-                                               font_desc,
-                                               &priv->font_iter);
+      gtk_font_chooser_widget_merge_font_desc (fontchooser, font_desc, &priv->font_iter);
                                                           
     }
 }
