@@ -12,9 +12,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -237,6 +235,9 @@ find_image_offset (GtkIconCache *cache,
   guint32 image_list_offset, n_images;
   int i;
 
+  if (!icon_name)
+    return 0;
+
   chain_offset = cache->last_chain_offset;
   if (chain_offset)
     {
@@ -299,6 +300,46 @@ _gtk_icon_cache_get_icon_flags (GtkIconCache *cache,
   return GET_UINT16 (cache->buffer, image_offset + 2);
 }
 
+gboolean
+_gtk_icon_cache_has_icons (GtkIconCache *cache,
+			   const gchar  *directory)
+{
+  int directory_index;
+  guint32 hash_offset, n_buckets;
+  guint32 chain_offset;
+  guint32 image_list_offset, n_images;
+  int i, j;
+
+  directory_index = get_directory_index (cache, directory);
+
+  if (directory_index == -1)
+    return FALSE;
+
+  hash_offset = GET_UINT32 (cache->buffer, 4);
+  n_buckets = GET_UINT32 (cache->buffer, hash_offset);
+
+  for (i = 0; i < n_buckets; i++)
+    {
+      chain_offset = GET_UINT32 (cache->buffer, hash_offset + 4 + 4 * i);
+      while (chain_offset != 0xffffffff)
+	{
+	  image_list_offset = GET_UINT32 (cache->buffer, chain_offset + 8);
+	  n_images = GET_UINT32 (cache->buffer, image_list_offset);
+
+	  for (j = 0; j < n_images; j++)
+	    {
+	      if (GET_UINT16 (cache->buffer, image_list_offset + 4 + 8 * j) ==
+		  directory_index)
+		return TRUE;
+	    }
+
+	  chain_offset = GET_UINT32 (cache->buffer, chain_offset);
+	}
+    }
+
+  return FALSE;
+}
+
 void
 _gtk_icon_cache_add_icons (GtkIconCache *cache,
 			   const gchar  *directory,
@@ -469,6 +510,7 @@ _gtk_icon_cache_get_icon (GtkIconCache *cache,
 
   length = GET_UINT32 (cache->buffer, pixel_data_offset + 4);
   
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   if (!gdk_pixdata_deserialize (&pixdata, length, 
 				(guchar *)(cache->buffer + pixel_data_offset + 8),
 				&error))
@@ -479,6 +521,7 @@ _gtk_icon_cache_get_icon (GtkIconCache *cache,
 
       return NULL;
     }
+G_GNUC_END_IGNORE_DEPRECATIONS
 
   pixbuf = gdk_pixbuf_new_from_data (pixdata.pixel_data, GDK_COLORSPACE_RGB,
 				     (pixdata.pixdata_type & GDK_PIXDATA_COLOR_TYPE_MASK) == GDK_PIXDATA_COLOR_TYPE_RGBA,
@@ -522,6 +565,7 @@ _gtk_icon_cache_get_icon_data  (GtkIconCache *cache,
     return NULL;
 
   data = g_slice_new0 (GtkIconData);
+  data->ref = 1;
 
   offset = GET_UINT32 (cache->buffer, meta_data_offset);
   if (offset)
