@@ -37,6 +37,26 @@
 #include "gtkprivate.h"
 #include "gtkintl.h"
 
+#ifdef GDK_WINDOWING_X11
+#include "x11/gdkx.h"
+#endif
+
+#ifdef GDK_WINDOWING_WAYLAND
+#include "wayland/gdkwayland.h"
+#endif
+
+#ifdef GDK_WINDOWING_BROADWAY
+#include "broadway/gdkbroadway.h"
+#endif
+
+#ifdef GDK_WINDOWING_WIN32
+#include "win32/gdkwin32.h"
+#endif
+
+#ifdef G_OS_WIN32
+#include <windows.h>
+#endif
+
 #undef GDK_DEPRECATED
 #undef GDK_DEPRECATED_FOR
 #define GDK_DEPRECATED
@@ -45,6 +65,7 @@
 #include "deprecated/gtkrc.h"
 
 #define SIMPLE_ID "gtk-im-context-simple"
+#define NONE_ID   "gtk-im-context-none"
 
 /**
  * GtkIMContextInfo:
@@ -53,7 +74,7 @@
  * @domain: Translation domain to be used with dgettext()
  * @domain_dirname: Name of locale directory for use with bindtextdomain()
  * @default_locales: A colon-separated list of locales where this input method
- *   should be the default. The asterisk "*" sets the default for all locales.
+ *   should be the default. The asterisk “*” sets the default for all locales.
  *
  * Bookkeeping information about a loadable input method.
  */
@@ -365,6 +386,9 @@ gtk_im_module_initialize (void)
 #ifdef INCLUDE_IM_xim
   do_builtin (xim);
 #endif
+#ifdef INCLUDE_IM_broadway
+  do_builtin (broadway);
+#endif
 
 #undef do_builtin
 
@@ -497,7 +521,7 @@ compare_gtkimcontextinfo_name(const GtkIMContextInfo **a,
  *            The structures it points are statically allocated and should
  *            not be modified or freed.
  * @n_contexts: the length of the array stored in @contexts
- * 
+ *
  * List all available types of input method context
  */
 void
@@ -512,7 +536,23 @@ _gtk_im_module_list (const GtkIMContextInfo ***contexts,
 #endif
 		GtkIMContextInfo simple_context_info = {
     SIMPLE_ID,
-    N_("Simple"),
+    NC_("input method menu", "Simple"),
+    GETTEXT_PACKAGE,
+#ifdef GTK_LOCALEDIR
+    GTK_LOCALEDIR,
+#else
+    "",
+#endif
+    ""
+  };
+
+  static
+#ifndef G_OS_WIN32
+	  const
+#endif
+		GtkIMContextInfo none_context_info = {
+    NONE_ID,
+    NC_("input method menu", "None"),
     GETTEXT_PACKAGE,
 #ifdef GTK_LOCALEDIR
     GTK_LOCALEDIR,
@@ -538,19 +578,22 @@ _gtk_im_module_list (const GtkIMContextInfo ***contexts,
        */
       simple_context_info.domain_dirname = g_strdup (simple_context_info.domain_dirname);
       correct_localedir_prefix ((char **) &simple_context_info.domain_dirname);
+      none_context_info.domain_dirname = g_strdup (none_context_info.domain_dirname);
+      correct_localedir_prefix ((char **) &none_context_info.domain_dirname);
     }
 #endif
 
   if (n_contexts)
-    *n_contexts = (n_loaded_contexts + 1);
+    *n_contexts = n_loaded_contexts + 2;
 
   if (contexts)
     {
       GSList *tmp_list;
       int i;
-      
-      *contexts = g_new (const GtkIMContextInfo *, n_loaded_contexts + 1);
 
+      *contexts = g_new (const GtkIMContextInfo *, n_loaded_contexts + 2);
+
+      (*contexts)[n++] = &none_context_info;
       (*contexts)[n++] = &simple_context_info;
 
       tmp_list = modules_list;
@@ -560,12 +603,12 @@ _gtk_im_module_list (const GtkIMContextInfo ***contexts,
 
 	  for (i=0; i<module->n_contexts; i++)
 	    (*contexts)[n++] = module->contexts[i];
-	  
+  
 	  tmp_list = tmp_list->next;
 	}
 
-      /* fisrt element (Default) should always be at top */
-      qsort ((*contexts)+1, n-1, sizeof (GtkIMContextInfo *), (GCompareFunc)compare_gtkimcontextinfo_name);
+      /* first elements (Simple and None) should always be at top */
+      qsort ((*contexts)+2, n-2, sizeof (GtkIMContextInfo *), (GCompareFunc)compare_gtkimcontextinfo_name);
     }
 }
 
@@ -576,7 +619,7 @@ _gtk_im_module_list (const GtkIMContextInfo ***contexts,
  * Create an IM context of a type specified by the string
  * ID @context_id.
  * 
- * Return value: a newly created input context of or @context_id, or
+ * Returns: a newly created input context of or @context_id, or
  *     if that could not be created, a newly created GtkIMContextSimple.
  */
 GtkIMContext *
@@ -584,7 +627,10 @@ _gtk_im_module_create (const gchar *context_id)
 {
   GtkIMModule *im_module;
   GtkIMContext *context = NULL;
-  
+
+  if (strcmp (context_id, NONE_ID) == 0)
+    return NULL;
+
   if (!contexts_hash)
     gtk_im_module_initialize ();
 
@@ -602,12 +648,12 @@ _gtk_im_module_create (const gchar *context_id)
 	      context = im_module->create (context_id);
 	      g_type_module_unuse (G_TYPE_MODULE (im_module));
 	    }
-	  
+
 	  if (!context)
 	    g_warning ("Loading IM context type '%s' failed", context_id);
 	}
     }
-  
+
   if (!context)
      return gtk_im_context_simple_new ();
   else
@@ -616,10 +662,10 @@ _gtk_im_module_create (const gchar *context_id)
 
 /* Match @locale against @against.
  * 
- * 'en_US' against 'en_US'       => 4
- * 'en_US' against 'en'          => 3
- * 'en', 'en_UK' against 'en_US' => 2
- *  all locales, against '*' 	 => 1
+ * 'en_US' against “en_US”       => 4
+ * 'en_US' against “en”          => 3
+ * 'en', “en_UK” against “en_US” => 2
+ *  all locales, against “*” 	 => 1
  */
 static gint
 match_locale (const gchar *locale,
@@ -638,6 +684,32 @@ match_locale (const gchar *locale,
   return 0;
 }
 
+static gboolean
+match_backend (GtkIMContextInfo *context)
+{
+#ifdef GDK_WINDOWING_WAYLAND
+  if (g_strcmp0 (context->context_id, "wayland") == 0)
+    return GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ());
+#endif
+
+#ifdef GDK_WINDOWING_BROADWAY
+  if (g_strcmp0 (context->context_id, "broadway") == 0)
+    return GDK_IS_BROADWAY_DISPLAY (gdk_display_get_default ());
+#endif
+
+#ifdef GDK_WINDOWING_X11
+  if (g_strcmp0 (context->context_id, "xim") == 0)
+    return GDK_IS_X11_DISPLAY (gdk_display_get_default ());
+#endif
+
+#ifdef GDK_WINDOWING_WIN32
+  if (g_strcmp0 (context->context_id, "ime") == 0)
+    return GDK_IS_WIN32_DISPLAY (gdk_display_get_default ());
+#endif
+
+  return TRUE;
+}
+
 static const gchar *
 lookup_immodule (gchar **immodules_list)
 {
@@ -645,19 +717,84 @@ lookup_immodule (gchar **immodules_list)
     {
       if (g_strcmp0 (*immodules_list, SIMPLE_ID) == 0)
         return SIMPLE_ID;
+      else if (g_strcmp0 (*immodules_list, NONE_ID) == 0)
+        return NONE_ID;
       else
-       {
-         GtkIMModule *module;
-         module = g_hash_table_lookup (contexts_hash, *immodules_list);
-         if (module)
-           return module->contexts[0]->context_id;
-       }
+	{
+	  gboolean found;
+	  gchar *context_id;
+	  found = g_hash_table_lookup_extended (contexts_hash, *immodules_list,
+						(gpointer *) &context_id, NULL);
+	  if (found)
+	    return context_id;
+	}
       immodules_list++;
     }
 
   return NULL;
 }
 
+#ifdef G_OS_WIN32
+
+/* max size for LOCALE_SISO639LANGNAME and LOCALE_SISO3166CTRYNAME is 9 */
+#define MAX_NAME_SIZE 9
+
+static gchar *
+get_current_input_language (void)
+{
+  LCID lcid;
+  LANGID langid;
+  HKL kblayout;
+  int name_size;
+  wchar_t name[MAX_NAME_SIZE];
+  gchar *language;
+  gchar *country;
+  gchar *full;
+
+  /* Current thread's keyboard layout */
+  kblayout = GetKeyboardLayout(0);
+  /* lowest word in the HKL is the LANGID */
+  langid = ((guint32)kblayout) & 0xFFFF;
+  /* LCID is the LANGID without order */
+  lcid = langid;
+
+  /* Get Language ID */
+  name_size = GetLocaleInfoW (lcid, LOCALE_SISO639LANGNAME, NULL, 0);
+  if (name_size <= 1)
+    return NULL;
+
+  g_assert (name_size <= MAX_NAME_SIZE);
+  GetLocaleInfoW (lcid, LOCALE_SISO639LANGNAME, name, name_size);
+
+  language = g_utf16_to_utf8 (name, name_size, NULL, NULL, NULL);
+  if (!language)
+    return NULL;
+
+  if (SUBLANGID (langid) == SUBLANG_NEUTRAL)
+    return language;
+
+  /* Get Country ID */
+  name_size = GetLocaleInfoW (lcid, LOCALE_SISO3166CTRYNAME, NULL, 0);
+  if (name_size <= 1)
+    return language;
+
+  g_assert (name_size <= MAX_NAME_SIZE);
+  GetLocaleInfoW (lcid, LOCALE_SISO3166CTRYNAME, name, name_size);
+
+  country = g_utf16_to_utf8 (name, name_size, NULL, NULL, NULL);
+  if (!country)
+    return language;
+
+  full = g_strdup_printf ("%s_%s", language, country);
+
+  g_free (language);
+  g_free (country);
+
+  return full;
+}
+
+#endif
+
 /**
  * _gtk_im_module_get_default_context_id:
  * @client_window: a window
@@ -665,7 +802,7 @@ lookup_immodule (gchar **immodules_list)
  * Return the context_id of the best IM context type 
  * for the given window.
  * 
- * Return value: the context ID (will never be %NULL)
+ * Returns: the context ID (will never be %NULL)
  */
 const gchar *
 _gtk_im_module_get_default_context_id (GdkWindow *client_window)
@@ -678,16 +815,16 @@ _gtk_im_module_get_default_context_id (GdkWindow *client_window)
   const gchar *envvar;
   GdkScreen *screen;
   GtkSettings *settings;
-      
+
   if (!contexts_hash)
     gtk_im_module_initialize ();
 
-  envvar = g_getenv("GTK_IM_MODULE");
+  envvar = g_getenv ("GTK_IM_MODULE");
   if (envvar)
     {
-        immodules = g_strsplit(envvar, ":", 0);
-        context_id = lookup_immodule(immodules);
-        g_strfreev(immodules);
+        immodules = g_strsplit (envvar, ":", 0);
+        context_id = lookup_immodule (immodules);
+        g_strfreev (immodules);
 
         if (context_id)
           return context_id;
@@ -695,41 +832,52 @@ _gtk_im_module_get_default_context_id (GdkWindow *client_window)
 
   /* Check if the certain immodule is set in XSETTINGS.
    */
-  if (GDK_IS_WINDOW (client_window))
+  screen = gdk_screen_get_default ();
+  settings = gtk_settings_get_for_screen (screen);
+  g_object_get (G_OBJECT (settings), "gtk-im-module", &tmp, NULL);
+  if (tmp)
     {
-      screen = gdk_window_get_screen (client_window);
-      settings = gtk_settings_get_for_screen (screen);
-      g_object_get (G_OBJECT (settings), "gtk-im-module", &tmp, NULL);
-      if (tmp)
-        {
-          immodules = g_strsplit(tmp, ":", 0);
-          context_id = lookup_immodule(immodules);
-          g_strfreev(immodules);
-          g_free (tmp);
+      immodules = g_strsplit (tmp, ":", 0);
+      context_id = lookup_immodule (immodules);
+      g_strfreev (immodules);
+      g_free (tmp);
 
-       	  if (context_id)
-            return context_id;
-        }
+      if (context_id)
+        return context_id;
     }
 
+#ifdef G_OS_WIN32
+  /* Read current input locale from the current keyboard info */
+  tmp_locale = get_current_input_language ();
+  if (!tmp_locale)
+    /* Default to system locale when input language is unknown */
+    tmp_locale = _gtk_get_lc_ctype ();
+#else
+  tmp_locale = _gtk_get_lc_ctype ();
+#endif
+
   /* Strip the locale code down to the essentials
    */
-  tmp_locale = _gtk_get_lc_ctype ();
   tmp = strchr (tmp_locale, '.');
   if (tmp)
     *tmp = '\0';
   tmp = strchr (tmp_locale, '@');
   if (tmp)
     *tmp = '\0';
-  
+
   tmp_list = modules_list;
   while (tmp_list)
     {
       GtkIMModule *module = tmp_list->data;
-     
+
       for (i = 0; i < module->n_contexts; i++)
 	{
-	  const gchar *p = module->contexts[i]->default_locales;
+	  const gchar *p;
+
+          if (!match_backend (module->contexts[i]))
+            continue;
+
+          p = module->contexts[i]->default_locales;
 	  while (p)
 	    {
 	      const gchar *q = strchr (p, ':');
@@ -744,11 +892,11 @@ _gtk_im_module_get_default_context_id (GdkWindow *client_window)
 	      p = q ? q + 1 : NULL;
 	    }
 	}
-      
+
       tmp_list = tmp_list->next;
     }
 
   g_free (tmp_locale);
-  
+
   return context_id ? context_id : SIMPLE_ID;
 }
