@@ -13,9 +13,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -29,32 +27,53 @@
 #include "gtkrecentchooserprivate.h"
 #include "gtkrecentchoosermenu.h"
 
-#include "gtkstock.h"
 #include "gtkicontheme.h"
-#include "gtkiconfactory.h"
 #include "gtkintl.h"
 #include "gtksettings.h"
 #include "gtkmenushell.h"
 #include "gtkmenuitem.h"
-#include "gtkimagemenuitem.h"
+#include "deprecated/gtkimagemenuitem.h"
 #include "gtkseparatormenuitem.h"
 #include "gtkmenu.h"
 #include "gtkimage.h"
 #include "gtklabel.h"
 #include "gtktooltip.h"
-#include "gtkactivatable.h"
+#include "deprecated/gtkactivatable.h"
 #include "gtktypebuiltins.h"
 #include "gtkprivate.h"
-#include "gtkalias.h"
+
+/**
+ * SECTION:gtkrecentchoosermenu
+ * @Short_description: Displays recently used files in a menu
+ * @Title: GtkRecentChooserMenu
+ * @See_also:#GtkRecentChooser
+ *
+ * #GtkRecentChooserMenu is a widget suitable for displaying recently used files
+ * inside a menu.  It can be used to set a sub-menu of a #GtkMenuItem using
+ * gtk_menu_item_set_submenu(), or as the menu of a #GtkMenuToolButton.
+ *
+ * Note that #GtkRecentChooserMenu does not have any methods of its own. Instead,
+ * you should use the functions that work on a #GtkRecentChooser.
+ *
+ * Note also that #GtkRecentChooserMenu does not support multiple filters, as it
+ * has no way to let the user choose between them as the #GtkRecentChooserWidget
+ * and #GtkRecentChooserDialog widgets do. Thus using gtk_recent_chooser_add_filter()
+ * on a #GtkRecentChooserMenu widget will yield the same effects as using
+ * gtk_recent_chooser_set_filter(), replacing any currently set filter
+ * with the supplied filter; gtk_recent_chooser_remove_filter() will remove
+ * any currently set #GtkRecentFilter object and will unset the current filter;
+ * gtk_recent_chooser_list_filters() will return a list containing a single
+ * #GtkRecentFilter object.
+ *
+ * Recently used files are supported since GTK+ 2.10.
+ */
+
 
 struct _GtkRecentChooserMenuPrivate
 {
   /* the recent manager object */
   GtkRecentManager *manager;
   
-  /* size of the icons of the menu items */  
-  gint icon_size;
-
   /* max size of the menu item label */
   gint label_width;
 
@@ -95,17 +114,13 @@ enum {
 };
 
 
-#define FALLBACK_ICON_SIZE 	32
 #define FALLBACK_ITEM_LIMIT 	10
 #define DEFAULT_LABEL_WIDTH     30
 
-#define GTK_RECENT_CHOOSER_MENU_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_RECENT_CHOOSER_MENU, GtkRecentChooserMenuPrivate))
 
 static void     gtk_recent_chooser_menu_finalize    (GObject                   *object);
 static void     gtk_recent_chooser_menu_dispose     (GObject                   *object);
-static GObject *gtk_recent_chooser_menu_constructor (GType                      type,
-						     guint                      n_construct_properties,
-						     GObjectConstructParam     *construct_params);
+static void     gtk_recent_chooser_menu_constructed (GObject                   *object);
 
 static void gtk_recent_chooser_iface_init      (GtkRecentChooserIface     *iface);
 
@@ -150,11 +165,6 @@ static void              gtk_recent_chooser_menu_set_show_tips      (GtkRecentCh
 static void     set_recent_manager (GtkRecentChooserMenu *menu,
 				    GtkRecentManager     *manager);
 
-static void     chooser_set_sort_type (GtkRecentChooserMenu *menu,
-				       GtkRecentSortType     sort_type);
-
-static gint     get_icon_size_for_widget (GtkWidget *widget);
-
 static void     item_activate_cb   (GtkWidget        *widget,
 			            gpointer          user_data);
 static void     manager_changed_cb (GtkRecentManager *manager,
@@ -167,14 +177,16 @@ static void gtk_recent_chooser_update                 (GtkActivatable       *act
 static void gtk_recent_chooser_sync_action_properties (GtkActivatable       *activatable,
 						       GtkAction            *action);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 G_DEFINE_TYPE_WITH_CODE (GtkRecentChooserMenu,
 			 gtk_recent_chooser_menu,
 			 GTK_TYPE_MENU,
+                         G_ADD_PRIVATE (GtkRecentChooserMenu)
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_RECENT_CHOOSER,
 				 		gtk_recent_chooser_iface_init)
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_ACTIVATABLE,
 				 		gtk_recent_chooser_activatable_iface_init))
-
+G_GNUC_END_IGNORE_DEPRECATIONS;
 
 static void
 gtk_recent_chooser_iface_init (GtkRecentChooserIface *iface)
@@ -205,7 +217,7 @@ gtk_recent_chooser_menu_class_init (GtkRecentChooserMenuClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  gobject_class->constructor = gtk_recent_chooser_menu_constructor;
+  gobject_class->constructed = gtk_recent_chooser_menu_constructed;
   gobject_class->dispose = gtk_recent_chooser_menu_dispose;
   gobject_class->finalize = gtk_recent_chooser_menu_finalize;
   gobject_class->set_property = gtk_recent_chooser_menu_set_property;
@@ -214,7 +226,7 @@ gtk_recent_chooser_menu_class_init (GtkRecentChooserMenuClass *klass)
   _gtk_recent_chooser_install_properties (gobject_class);
 
   /**
-   * GtkRecentChooserMenu:show-numbers
+   * GtkRecentChooserMenu:show-numbers:
    *
    * Whether the first ten items in the menu should be prepended by
    * a number acting as a unique mnemonic.
@@ -227,24 +239,21 @@ gtk_recent_chooser_menu_class_init (GtkRecentChooserMenuClass *klass)
 							 P_("Show Numbers"),
 							 P_("Whether the items should be displayed with a number"),
 							 FALSE,
-							 GTK_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
   
 
   g_object_class_override_property (gobject_class, PROP_ACTIVATABLE_RELATED_ACTION, "related-action");
   g_object_class_override_property (gobject_class, PROP_ACTIVATABLE_USE_ACTION_APPEARANCE, "use-action-appearance");
-
-  g_type_class_add_private (klass, sizeof (GtkRecentChooserMenuPrivate));
 }
 
 static void
 gtk_recent_chooser_menu_init (GtkRecentChooserMenu *menu)
 {
   GtkRecentChooserMenuPrivate *priv;
-  
-  priv = GTK_RECENT_CHOOSER_MENU_GET_PRIVATE (menu);
-  
-  menu->priv = priv;
-  
+
+  menu->priv = gtk_recent_chooser_menu_get_instance_private (menu);
+  priv = menu->priv;
+
   priv->show_icons= TRUE;
   priv->show_numbers = FALSE;
   priv->show_tips = FALSE;
@@ -254,7 +263,6 @@ gtk_recent_chooser_menu_init (GtkRecentChooserMenu *menu)
   
   priv->limit = FALLBACK_ITEM_LIMIT;
   priv->sort_type = GTK_RECENT_SORT_NONE;
-  priv->icon_size = FALLBACK_ICON_SIZE;
   priv->label_width = DEFAULT_LABEL_WIDTH;
   
   priv->first_recent_item_pos = -1;
@@ -312,20 +320,13 @@ gtk_recent_chooser_menu_dispose (GObject *object)
   G_OBJECT_CLASS (gtk_recent_chooser_menu_parent_class)->dispose (object);
 }
 
-static GObject *
-gtk_recent_chooser_menu_constructor (GType                  type,
-				     guint                  n_params,
-				     GObjectConstructParam *params)
+static void
+gtk_recent_chooser_menu_constructed (GObject *object)
 {
-  GtkRecentChooserMenu *menu;
-  GtkRecentChooserMenuPrivate *priv;
-  GObjectClass *parent_class;
-  GObject *object;
-  
-  parent_class = G_OBJECT_CLASS (gtk_recent_chooser_menu_parent_class);
-  object = parent_class->constructor (type, n_params, params);
-  menu = GTK_RECENT_CHOOSER_MENU (object);
-  priv = menu->priv;
+  GtkRecentChooserMenu *menu = GTK_RECENT_CHOOSER_MENU (object);
+  GtkRecentChooserMenuPrivate *priv = menu->priv;
+
+  G_OBJECT_CLASS (gtk_recent_chooser_menu_parent_class)->constructed (object);
   
   g_assert (priv->manager);
 
@@ -351,8 +352,6 @@ gtk_recent_chooser_menu_constructor (GType                  type,
 
   /* (re)populate the menu */
   gtk_recent_chooser_menu_populate (menu);
-
-  return object;
 }
 
 static void
@@ -367,22 +366,38 @@ gtk_recent_chooser_menu_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_SHOW_NUMBERS:
-      priv->show_numbers = g_value_get_boolean (value);
+      if (priv->show_numbers != g_value_get_boolean (value))
+        {
+          priv->show_numbers = g_value_get_boolean (value);
+          g_object_notify_by_pspec (object, pspec);
+        }
       break;
     case GTK_RECENT_CHOOSER_PROP_RECENT_MANAGER:
       set_recent_manager (menu, g_value_get_object (value));
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_PRIVATE:
-      priv->show_private = g_value_get_boolean (value);
+      if (priv->show_private != g_value_get_boolean (value))
+        {
+          priv->show_private = g_value_get_boolean (value);
+          g_object_notify_by_pspec (object, pspec);
+        }
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_NOT_FOUND:
-      priv->show_not_found = g_value_get_boolean (value);
+      if (priv->show_not_found != g_value_get_boolean (value))
+        {
+          priv->show_not_found = g_value_get_boolean (value);
+          g_object_notify_by_pspec (object, pspec);
+        }
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_TIPS:
       gtk_recent_chooser_menu_set_show_tips (menu, g_value_get_boolean (value));
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_ICONS:
-      priv->show_icons = g_value_get_boolean (value);
+      if (priv->show_icons != g_value_get_boolean (value))
+        {
+          priv->show_icons = g_value_get_boolean (value);
+          g_object_notify_by_pspec (object, pspec);
+        }
       break;
     case GTK_RECENT_CHOOSER_PROP_SELECT_MULTIPLE:
       g_warning ("%s: Choosers of type `%s' do not support selecting multiple items.",
@@ -390,13 +405,25 @@ gtk_recent_chooser_menu_set_property (GObject      *object,
                  G_OBJECT_TYPE_NAME (object));
       break;
     case GTK_RECENT_CHOOSER_PROP_LOCAL_ONLY:
-      priv->local_only = g_value_get_boolean (value);
+      if (priv->local_only != g_value_get_boolean (value))
+        {
+          priv->local_only = g_value_get_boolean (value);
+          g_object_notify_by_pspec (object, pspec);
+        }
       break;
     case GTK_RECENT_CHOOSER_PROP_LIMIT:
-      priv->limit = g_value_get_int (value);
+      if (priv->limit != g_value_get_int (value))
+        {
+          priv->limit = g_value_get_int (value);
+          g_object_notify_by_pspec (object, pspec);
+        }
       break;
     case GTK_RECENT_CHOOSER_PROP_SORT_TYPE:
-      chooser_set_sort_type (menu, g_value_get_enum (value));
+      if (priv->sort_type != g_value_get_enum (value))
+        {
+          priv->sort_type = g_value_get_enum (value);
+          g_object_notify_by_pspec (object, pspec);
+        } 
       break;
     case GTK_RECENT_CHOOSER_PROP_FILTER:
       gtk_recent_chooser_menu_set_current_filter (menu, g_value_get_object (value));
@@ -475,10 +502,11 @@ gtk_recent_chooser_menu_set_current_uri (GtkRecentChooser  *chooser,
   GList *children, *l;
   GtkWidget *menu_item = NULL;
   gboolean found = FALSE;
+  gint i = 0;
   
   children = gtk_container_get_children (GTK_CONTAINER (menu));
   
-  for (l = children; l != NULL; l = l->next)
+  for (l = children; l != NULL; l = l->next, i++)
     {
       GtkRecentInfo *info;
       
@@ -490,9 +518,7 @@ gtk_recent_chooser_menu_set_current_uri (GtkRecentChooser  *chooser,
       
       if (strcmp (uri, gtk_recent_info_get_uri (info)) == 0)
         {
-          gtk_menu_shell_activate_item (GTK_MENU_SHELL (menu),
-	                                menu_item,
-					TRUE);
+          gtk_menu_set_active (GTK_MENU (menu), i);
 	  found = TRUE;
 
 	  break;
@@ -505,7 +531,7 @@ gtk_recent_chooser_menu_set_current_uri (GtkRecentChooser  *chooser,
     {
       g_set_error (error, GTK_RECENT_CHOOSER_ERROR,
       		   GTK_RECENT_CHOOSER_ERROR_NOT_FOUND,
-      		   _("No recently used resource found with URI `%s'"),
+                   _("No recently used resource found with URI '%s'"),
       		   uri);
     }
   
@@ -561,7 +587,7 @@ gtk_recent_chooser_menu_select_uri (GtkRecentChooser  *chooser,
     {
       g_set_error (error, GTK_RECENT_CHOOSER_ERROR,
       		   GTK_RECENT_CHOOSER_ERROR_NOT_FOUND,
-      		   _("No recently used resource found with URI `%s'"),
+                   _("No recently used resource found with URI '%s'"),
       		   uri);
       return FALSE;
     }
@@ -585,16 +611,16 @@ gtk_recent_chooser_menu_unselect_uri (GtkRecentChooser *chooser,
 static void
 gtk_recent_chooser_menu_select_all (GtkRecentChooser *chooser)
 {
-  g_warning (_("This function is not implemented for "
-               "widgets of class '%s'"),
+  g_warning ("This function is not implemented for "
+	     "widgets of class '%s'",
              g_type_name (G_OBJECT_TYPE (chooser)));
 }
 
 static void
 gtk_recent_chooser_menu_unselect_all (GtkRecentChooser *chooser)
 {
-  g_warning (_("This function is not implemented for "
-               "widgets of class '%s'"),
+  g_warning ("This function is not implemented for "
+	     "widgets of class '%s'",
              g_type_name (G_OBJECT_TYPE (chooser)));
 }
 
@@ -625,17 +651,6 @@ gtk_recent_chooser_menu_set_sort_func (GtkRecentChooser  *chooser,
     }
 }
 
-static void
-chooser_set_sort_type (GtkRecentChooserMenu *menu,
-		       GtkRecentSortType     sort_type)
-{
-  if (menu->priv->sort_type == sort_type)
-    return;
-
-  menu->priv->sort_type = sort_type;
-}
-
-
 static GList *
 gtk_recent_chooser_menu_get_items (GtkRecentChooser *chooser)
 {
@@ -789,7 +804,7 @@ gtk_recent_chooser_menu_create_item (GtkRecentChooserMenu *menu,
   GtkRecentChooserMenuPrivate *priv;
   gchar *text;
   GtkWidget *item, *image, *label;
-  GdkPixbuf *icon;
+  GIcon *icon;
 
   g_assert (info != NULL);
 
@@ -818,27 +833,33 @@ gtk_recent_chooser_menu_create_item (GtkRecentChooserMenu *menu,
          * The %d is the number of the item, the %s is the name of the item.
          */
         text = g_strdup_printf (C_("recent menu label", "%d. %s"), count, escaped);
-      
+
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       item = gtk_image_menu_item_new_with_mnemonic (text);
-      
+      G_GNUC_END_IGNORE_DEPRECATIONS;
+
       g_free (escaped);
       g_free (name);
     }
   else
     {
       text = g_strdup (gtk_recent_info_get_display_name (info));
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       item = gtk_image_menu_item_new_with_label (text);
+      G_GNUC_END_IGNORE_DEPRECATIONS;
     }
 
   g_free (text);
 
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
   gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (item),
                                              TRUE);
+  G_GNUC_END_IGNORE_DEPRECATIONS;
 
   /* ellipsize the menu item label, in case the recent document
    * display name is huge.
    */
-  label = GTK_BIN (item)->child;
+  label = gtk_bin_get_child (GTK_BIN (item));
   if (GTK_IS_LABEL (label))
     {
       gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
@@ -847,11 +868,15 @@ gtk_recent_chooser_menu_create_item (GtkRecentChooserMenu *menu,
   
   if (priv->show_icons)
     {
-      icon = gtk_recent_info_get_icon (info, priv->icon_size);
-        
-      image = gtk_image_new_from_pixbuf (icon);
+      icon = gtk_recent_info_get_gicon (info);
+
+      image = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_MENU);
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-      g_object_unref (icon);
+      gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (item), TRUE);
+      G_GNUC_END_IGNORE_DEPRECATIONS;
+      if (icon)
+        g_object_unref (icon);
     }
 
   g_signal_connect (item, "activate",
@@ -947,6 +972,30 @@ typedef struct
   GtkWidget *placeholder;
 } MenuPopulateData;
 
+static MenuPopulateData *
+create_menu_populate_data (GtkRecentChooserMenu *menu)
+{
+  MenuPopulateData *pdata;
+
+  pdata = g_slice_new (MenuPopulateData);
+  pdata->items = NULL;
+  pdata->n_items = 0;
+  pdata->loaded_items = 0;
+  pdata->displayed_items = 0;
+  pdata->menu = menu;
+  pdata->placeholder = g_object_ref (menu->priv->placeholder);
+
+  return pdata;
+}
+
+static void
+free_menu_populate_data (MenuPopulateData *pdata)
+{
+  if (pdata->placeholder)
+    g_object_unref (pdata->placeholder);
+  g_slice_free (MenuPopulateData, pdata);
+}
+
 static gboolean
 idle_populate_func (gpointer data)
 {
@@ -1009,8 +1058,7 @@ check_and_return:
 
   if (pdata->loaded_items == pdata->n_items)
     {
-      g_list_foreach (pdata->items, (GFunc) gtk_recent_info_unref, NULL);
-      g_list_free (pdata->items);
+      g_list_free_full (pdata->items, (GDestroyNotify) gtk_recent_info_unref);
 
       priv->populate_id = 0;
 
@@ -1034,10 +1082,9 @@ idle_populate_clean_up (gpointer data)
        */
       if (!pdata->displayed_items)
         gtk_widget_show (pdata->placeholder);
-      g_object_unref (pdata->placeholder);
-
-      g_slice_free (MenuPopulateData, data);
     }
+
+  free_menu_populate_data (pdata);
 }
 
 static void
@@ -1046,26 +1093,19 @@ gtk_recent_chooser_menu_populate (GtkRecentChooserMenu *menu)
   MenuPopulateData *pdata;
   GtkRecentChooserMenuPrivate *priv = menu->priv;
 
-  if (menu->priv->populate_id)
+  if (priv->populate_id)
     return;
 
-  pdata = g_slice_new (MenuPopulateData);
-  pdata->items = NULL;
-  pdata->n_items = 0;
-  pdata->loaded_items = 0;
-  pdata->displayed_items = 0;
-  pdata->menu = menu;
-  pdata->placeholder = g_object_ref (priv->placeholder);
+  pdata = create_menu_populate_data (menu);
 
-  priv->icon_size = get_icon_size_for_widget (GTK_WIDGET (menu));
-  
   /* remove our menu items first */
   gtk_recent_chooser_menu_dispose_items (menu);
-  
+
   priv->populate_id = gdk_threads_add_idle_full (G_PRIORITY_HIGH_IDLE + 30,
   					         idle_populate_func,
 					         pdata,
                                                  idle_populate_clean_up);
+  g_source_set_name_by_id (priv->populate_id, "[gtk+] idle_populate_func");
 }
 
 /* bounce activate signal from the recent menu item widget 
@@ -1076,7 +1116,9 @@ item_activate_cb (GtkWidget *widget,
 		  gpointer   user_data)
 {
   GtkRecentChooser *chooser = GTK_RECENT_CHOOSER (user_data);
-  
+  GtkRecentInfo *info = g_object_get_data (G_OBJECT (widget), "gtk-recent-info");
+
+  gtk_recent_chooser_menu_set_current_uri (chooser, gtk_recent_info_get_uri (info), NULL);
   _gtk_recent_chooser_item_activated (chooser);
 }
 
@@ -1124,24 +1166,6 @@ set_recent_manager (GtkRecentChooserMenu *menu,
                                                  menu);
 }
 
-static gint
-get_icon_size_for_widget (GtkWidget *widget)
-{
-  GtkSettings *settings;
-  gint width, height;
-
-  if (gtk_widget_has_screen (widget))
-    settings = gtk_settings_get_for_screen (gtk_widget_get_screen (widget));
-  else
-    settings = gtk_settings_get_default ();
-
-  if (gtk_icon_size_lookup_for_settings (settings, GTK_ICON_SIZE_MENU,
-                                         &width, &height))
-    return MAX (width, height);
-
-  return FALLBACK_ICON_SIZE;
-}
-
 static void
 foreach_set_shot_tips (GtkWidget *widget,
                        gpointer   user_data)
@@ -1169,6 +1193,7 @@ gtk_recent_chooser_menu_set_show_tips (GtkRecentChooserMenu *menu,
   
   priv->show_tips = show_tips;
   gtk_container_foreach (GTK_CONTAINER (menu), foreach_set_shot_tips, menu);
+  g_object_notify (G_OBJECT (menu), "show-tips");
 }
 
 static void
@@ -1176,9 +1201,13 @@ gtk_recent_chooser_update (GtkActivatable *activatable,
 			   GtkAction      *action,
 			   const gchar    *property_name)
 {
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
+
   if (strcmp (property_name, "sensitive") == 0)
     gtk_widget_set_sensitive (GTK_WIDGET (activatable), gtk_action_is_sensitive (action));
 
+  G_GNUC_END_IGNORE_DEPRECATIONS;
+
   _gtk_recent_chooser_update (activatable, action, property_name);
 }
 
@@ -1186,8 +1215,15 @@ static void
 gtk_recent_chooser_sync_action_properties (GtkActivatable *activatable,
 				           GtkAction      *action)
 {
+  if (!action)
+    return;
+
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
+
   gtk_widget_set_sensitive (GTK_WIDGET (activatable), gtk_action_is_sensitive (action));
 
+  G_GNUC_END_IGNORE_DEPRECATIONS;
+
   _gtk_recent_chooser_sync_action_properties (activatable, action);
 }
 
@@ -1212,7 +1248,7 @@ gtk_recent_chooser_sync_action_properties (GtkActivatable *activatable,
  * gtk_recent_chooser_menu_new_for_manager() function to know how to create
  * a #GtkRecentChooserMenu widget bound to another #GtkRecentManager object.
  *
- * Return value: a new #GtkRecentChooserMenu
+ * Returns: a new #GtkRecentChooserMenu
  *
  * Since: 2.10
  */
@@ -1236,7 +1272,7 @@ gtk_recent_chooser_menu_new (void)
  * object or if you wish to share a common #GtkRecentManager object
  * among multiple #GtkRecentChooser widgets.
  *
- * Return value: a new #GtkRecentChooserMenu, bound to @manager.
+ * Returns: a new #GtkRecentChooserMenu, bound to @manager.
  *
  * Since: 2.10
  */
@@ -1256,7 +1292,7 @@ gtk_recent_chooser_menu_new_for_manager (GtkRecentManager *manager)
  *
  * Returns the value set by gtk_recent_chooser_menu_set_show_numbers().
  * 
- * Return value: %TRUE if numbers should be shown.
+ * Returns: %TRUE if numbers should be shown.
  *
  * Since: 2.10
  */
@@ -1275,7 +1311,7 @@ gtk_recent_chooser_menu_get_show_numbers (GtkRecentChooserMenu *menu)
  *
  * Sets whether a number should be added to the items of @menu.  The
  * numbers are shown to provide a unique character for a mnemonic to
- * be used inside ten menu item's label.  Only the first the items
+ * be used inside ten menu item’s label.  Only the first the items
  * get a number to avoid clashes.
  *
  * Since: 2.10
@@ -1292,6 +1328,3 @@ gtk_recent_chooser_menu_set_show_numbers (GtkRecentChooserMenu *menu,
   menu->priv->show_numbers = show_numbers;
   g_object_notify (G_OBJECT (menu), "show-numbers");
 }
-
-#define __GTK_RECENT_CHOOSER_MENU_C__
-#include "gtkaliasdef.c"
