@@ -101,12 +101,15 @@ _gtk_text_handle_draw (GtkTextHandle         *handle,
   gint width, height;
 
   priv = handle->priv;
-  context = gtk_widget_get_style_context (priv->parent);
+  context = gtk_widget_get_style_context (priv->windows[pos].widget);
+  _gtk_text_handle_get_size (handle, &width, &height);
 
   cairo_save (cr);
 
   if (pos == GTK_TEXT_HANDLE_POSITION_SELECTION_END)
-    cairo_translate (cr, 0, priv->windows[pos].pointing_to.height);
+    cairo_translate (cr, width, priv->windows[pos].pointing_to.height);
+  else
+    cairo_translate (cr, width, height);
 
   gtk_style_context_save (context);
   gtk_style_context_add_class (context,
@@ -125,8 +128,9 @@ _gtk_text_handle_draw (GtkTextHandle         *handle,
     gtk_style_context_add_class (context,
                                  GTK_STYLE_CLASS_TOP);
 
-  _gtk_text_handle_get_size (handle, &width, &height);
   gtk_render_background (context, cr, 0, 0, width, height);
+  gtk_render_frame (context, cr, 0, 0, width, height);
+  gtk_render_handle (context, cr, 0, 0, width, height);
 
   gtk_style_context_restore (context);
   cairo_restore (cr);
@@ -162,6 +166,25 @@ gtk_text_handle_widget_draw (GtkWidget     *widget,
   return TRUE;
 }
 
+static void
+gtk_text_handle_set_state (GtkTextHandle *handle,
+                           GtkStateFlags  state)
+{
+  GtkTextHandlePrivate *priv;
+  gint i;
+
+  priv = handle->priv;
+
+  for (i = 0; i <= GTK_TEXT_HANDLE_POSITION_SELECTION_START; i++)
+    {
+      if (!priv->windows[i].widget)
+        continue;
+
+      gtk_widget_set_state_flags (priv->windows[i].widget, state, TRUE);
+      gtk_widget_queue_draw (priv->windows[i].widget);
+    }
+}
+
 static gboolean
 gtk_text_handle_widget_event (GtkWidget     *widget,
                               GdkEvent      *event,
@@ -181,24 +204,30 @@ gtk_text_handle_widget_event (GtkWidget     *widget,
       priv->windows[pos].dx = event->button.x;
       priv->windows[pos].dy = event->button.y;
       priv->windows[pos].dragged = TRUE;
+      gtk_text_handle_set_state (handle, GTK_STATE_FLAG_ACTIVE);
     }
   else if (event->type == GDK_BUTTON_RELEASE)
     {
       g_signal_emit (handle, signals[DRAG_FINISHED], 0, pos);
       priv->windows[pos].dragged = FALSE;
+      gtk_text_handle_set_state (handle, GTK_STATE_FLAG_NORMAL);
     }
   else if (event->type == GDK_MOTION_NOTIFY &&
            event->motion.state & GDK_BUTTON1_MASK &&
            priv->windows[pos].dragged)
     {
-      gint x, y, width, height;
+      gint x, y, width, height, handle_height;
+      GtkAllocation allocation;
 
-      _gtk_text_handle_get_size (handle, &width, &height);
+      gtk_widget_get_allocation (priv->windows[pos].widget, &allocation);
+      width = allocation.width;
+      height = allocation.height;
+      _gtk_text_handle_get_size (handle, NULL, &handle_height);
       x = event->motion.x - priv->windows[pos].dx + (width / 2);
       y = event->motion.y - priv->windows[pos].dy;
 
       if (pos != GTK_TEXT_HANDLE_POSITION_CURSOR)
-        y += height;
+        y += height - handle_height;
 
       y += priv->windows[pos].pointing_to.height / 2;
 
@@ -213,6 +242,12 @@ static void
 gtk_text_handle_widget_style_updated (GtkWidget     *widget,
                                       GtkTextHandle *handle)
 {
+  GtkTextHandlePrivate *priv;
+
+  priv = handle->priv;
+  gtk_style_context_set_parent (gtk_widget_get_style_context (widget),
+                                gtk_widget_get_style_context (priv->parent));
+
   _gtk_text_handle_update (handle, GTK_TEXT_HANDLE_POSITION_SELECTION_START);
   _gtk_text_handle_update (handle, GTK_TEXT_HANDLE_POSITION_SELECTION_END);
 }
@@ -246,6 +281,9 @@ _gtk_text_handle_ensure_widget (GtkTextHandle         *handle,
       priv->windows[pos].widget = g_object_ref_sink (widget);
       window = gtk_widget_get_ancestor (priv->parent, GTK_TYPE_WINDOW);
       _gtk_window_add_popover (GTK_WINDOW (window), widget);
+
+      gtk_style_context_set_parent (gtk_widget_get_style_context (widget),
+                                    gtk_widget_get_style_context (priv->parent));
     }
 
   return priv->windows[pos].widget;
@@ -313,6 +351,13 @@ _gtk_text_handle_update (GtkTextHandle         *handle,
       rect.width = width;
       rect.height = 0;
 
+      /* Make the window 3 times as wide, and 2 times as high (plus
+       * handle_window->pointing_to.height), the handle will be rendered
+       * in the center. Making the rest an invisible input area.
+       */
+      width *= 3;
+      height *= 2;
+
       _handle_update_child_visible (handle, pos);
 
       window = gtk_widget_get_parent (handle_window->widget);
