@@ -20,6 +20,7 @@
 #include "config.h"
 
 #include "gtktooltip.h"
+#include "gtktooltipprivate.h"
 
 #include <math.h>
 #include <string.h>
@@ -34,11 +35,15 @@
 #include "gtksizerequest.h"
 #include "gtkstylecontext.h"
 #include "gtkwindowprivate.h"
+#include "gtkaccessible.h"
 
 
 #ifdef GDK_WINDOWING_WAYLAND
 #include "wayland/gdkwayland.h"
 #endif
+#ifdef GDK_WINDOWING_MIR
+#include "mir/gdkmir.h"
+#endif
 
 
 /**
@@ -109,7 +114,6 @@ struct _GtkTooltip
   GtkWidget *keyboard_widget;
 
   GtkWidget *tooltip_widget;
-  GdkWindow *toplevel_window;
 
   gdouble last_x;
   gdouble last_y;
@@ -138,12 +142,6 @@ static void       gtk_tooltip_class_init           (GtkTooltipClass *klass);
 static void       gtk_tooltip_init                 (GtkTooltip      *tooltip);
 static void       gtk_tooltip_dispose              (GObject         *object);
 
-static gboolean   gtk_tooltip_paint_window         (GtkTooltip      *tooltip,
-                                                    cairo_t         *cr);
-static void       gtk_tooltip_realize_window       (GtkTooltip      *tooltip,
-                                                    GtkWidget       *widget);
-static void       gtk_tooltip_composited_changed   (GtkTooltip      *tooltip,
-                                                    GtkWidget       *widget);
 static void       gtk_tooltip_window_hide          (GtkWidget       *widget,
 						    gpointer         user_data);
 static void       gtk_tooltip_display_closed       (GdkDisplay      *display,
@@ -173,8 +171,7 @@ gtk_tooltip_init (GtkTooltip *tooltip)
   GtkWidget *box;
   GtkWidget *image;
   GtkWidget *label;
-  GdkScreen *screen;
-  GdkVisual *visual;
+  AtkObject *atk_obj;
 
   tooltip->timeout_id = 0;
   tooltip->browse_mode_timeout_id = 0;
@@ -186,33 +183,23 @@ gtk_tooltip_init (GtkTooltip *tooltip)
   tooltip->keyboard_widget = NULL;
 
   tooltip->tooltip_widget = NULL;
-  tooltip->toplevel_window = NULL;
 
   tooltip->last_window = NULL;
 
   window = gtk_window_new (GTK_WINDOW_POPUP);
-  screen = gtk_widget_get_screen (window);
-  visual = gdk_screen_get_rgba_visual (screen);
-
-  if (visual != NULL)
-    gtk_widget_set_visual (window, visual);
-
   gtk_window_set_type_hint (GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_TOOLTIP);
-  gtk_widget_set_app_paintable (window, TRUE);
   gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
-  gtk_widget_set_name (window, "gtk-tooltip");
+  gtk_window_set_use_subsurface (GTK_WINDOW (window), TRUE);
   g_signal_connect (window, "hide",
                     G_CALLBACK (gtk_tooltip_window_hide), tooltip);
 
+  _gtk_window_request_csd (GTK_WINDOW (window));
   context = gtk_widget_get_style_context (window);
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_TOOLTIP);
 
-  g_signal_connect_swapped (window, "draw",
-                            G_CALLBACK (gtk_tooltip_paint_window), tooltip);
-  g_signal_connect_swapped (window, "realize",
-                            G_CALLBACK (gtk_tooltip_realize_window), tooltip);
-  g_signal_connect_swapped (window, "composited-changed",
-                            G_CALLBACK (gtk_tooltip_composited_changed), tooltip);
+  atk_obj = gtk_widget_get_accessible (window);
+  if (GTK_IS_ACCESSIBLE (atk_obj))
+    atk_object_set_role (atk_obj, ATK_ROLE_TOOL_TIP);
 
   /* FIXME: don't hardcode the padding */
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
@@ -228,6 +215,7 @@ gtk_tooltip_init (GtkTooltip *tooltip)
 
   label = gtk_label_new ("");
   gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+  gtk_label_set_max_width_chars (GTK_LABEL (label), 70);
   gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
 
   tooltip->window = window;
@@ -589,88 +577,6 @@ gtk_tooltip_reset (GtkTooltip *tooltip)
   tooltip->custom_was_reset = FALSE;
 }
 
-static void
-paint_background_and_frame (GtkTooltip *tooltip,
-                            cairo_t *cr)
-{
-  GtkStyleContext *context;
-  gint width, height;
-
-  width = gtk_widget_get_allocated_width (tooltip->window);
-  height = gtk_widget_get_allocated_height (tooltip->window);
-  context = gtk_widget_get_style_context (tooltip->window);
-
-  gtk_render_background (context, cr,
-                         0, 0, width, height);
-  gtk_render_frame (context, cr,
-                    0, 0, width, height);  
-}
-
-static void
-maybe_update_shape (GtkTooltip *tooltip)
-{
-  cairo_t *cr;
-  cairo_surface_t *surface;
-  cairo_region_t *region;
-
-  /* fallback to XShape only for non-composited clients */
-  if (gtk_widget_is_composited (tooltip->window))
-    {
-      gtk_widget_shape_combine_region (tooltip->window, NULL);
-      return;
-    }
-
-  surface = gdk_window_create_similar_surface (gtk_widget_get_window (tooltip->window),
-                                               CAIRO_CONTENT_COLOR_ALPHA,
-                                               gtk_widget_get_allocated_width (tooltip->window),
-                                               gtk_widget_get_allocated_height (tooltip->window));
-
-  cr = cairo_create (surface);
-  paint_background_and_frame (tooltip, cr);
-  cairo_destroy (cr);
-
-  region = gdk_cairo_region_create_from_surface (surface);
-  gtk_widget_shape_combine_region (tooltip->window, region);
-
-  cairo_surface_destroy (surface);
-  cairo_region_destroy (region);
-}
-
-static void
-gtk_tooltip_composited_changed (GtkTooltip *tooltip,
-                                GtkWidget  *widget)
-{
-  if (gtk_widget_get_realized (tooltip->window))
-    maybe_update_shape (tooltip);
-}
-
-static void
-gtk_tooltip_realize_window (GtkTooltip *tooltip,
-                            GtkWidget *widget)
-{
-  maybe_update_shape (tooltip);
-}
-
-static gboolean
-gtk_tooltip_paint_window (GtkTooltip *tooltip,
-                          cairo_t    *cr)
-{
-  if (gtk_widget_is_composited (tooltip->window))
-    {
-      /* clear any background */
-      cairo_save (cr);
-      cairo_set_source_rgba (cr, 0, 0, 0, 0);
-      cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-      cairo_paint (cr);
-      cairo_restore (cr);
-    }
-
-  maybe_update_shape (tooltip);
-  paint_background_and_frame (tooltip, cr);
-
-  return FALSE;
-}
-
 static void
 gtk_tooltip_window_hide (GtkWidget *widget,
 			 gpointer   user_data)
@@ -934,8 +840,20 @@ find_topmost_widget_coords_from_event (GdkEvent *event,
 
   /* Make sure the pointer can actually be on the widget returned. */
   gtk_widget_get_allocation (tmp, &allocation);
-  if (tx < 0 || tx >= allocation.width ||
-      ty < 0 || ty >= allocation.height)
+  allocation.x = 0;
+  allocation.y = 0;
+  if (GTK_IS_WINDOW (tmp))
+    {
+      GtkBorder border;
+      _gtk_window_get_shadow_width (GTK_WINDOW (tmp), &border);
+      allocation.x = border.left;
+      allocation.y = border.top;
+      allocation.width -= border.left + border.right;
+      allocation.height -= border.top + border.bottom;
+    }
+
+  if (tx < allocation.x || tx >= allocation.width ||
+      ty < allocation.y || ty >= allocation.height)
     return NULL;
 
   if (x)
@@ -975,6 +893,8 @@ static void
 gtk_tooltip_set_last_window (GtkTooltip *tooltip,
 			     GdkWindow  *window)
 {
+  GtkWidget *window_widget = NULL;
+
   if (tooltip->last_window == window)
     return;
 
@@ -984,9 +904,24 @@ gtk_tooltip_set_last_window (GtkTooltip *tooltip,
 
   tooltip->last_window = window;
 
-  if (window)
+  if (tooltip->last_window)
     g_object_add_weak_pointer (G_OBJECT (tooltip->last_window),
 			       (gpointer *) &tooltip->last_window);
+
+  if (window)
+    gdk_window_get_user_data (window, (gpointer *) &window_widget);
+
+  if (window_widget)
+    window_widget = gtk_widget_get_toplevel (window_widget);
+
+  if (window_widget &&
+      window_widget != tooltip->window &&
+      gtk_widget_is_toplevel (window_widget) &&
+      GTK_IS_WINDOW (window_widget))
+    gtk_window_set_transient_for (GTK_WINDOW (tooltip->window),
+                                  GTK_WINDOW (window_widget));
+  else
+    gtk_window_set_transient_for (GTK_WINDOW (tooltip->window), NULL);
 }
 
 static gboolean
@@ -1042,6 +977,7 @@ get_bounding_box (GtkWidget    *widget,
                   GdkRectangle *bounds)
 {
   GtkAllocation allocation;
+  GtkBorder border = { 0, };
   GdkWindow *window;
   gint x, y;
   gint w, h;
@@ -1055,10 +991,12 @@ get_bounding_box (GtkWidget    *widget,
     window = gtk_widget_get_window (widget);
 
   gtk_widget_get_allocation (widget, &allocation);
-  x = allocation.x;
-  y = allocation.y;
-  w = allocation.width;
-  h = allocation.height;
+  if (GTK_IS_WINDOW (widget))
+    _gtk_window_get_shadow_width (GTK_WINDOW (widget), &border);
+  x = allocation.x + border.left;
+  y = allocation.y + border.right;
+  w = allocation.width - border.left - border.right;
+  h = allocation.height - border.top - border.bottom;
 
   gdk_window_get_root_coords (window, x, y, &x1, &y1);
   gdk_window_get_root_coords (window, x + w, y, &x2, &y2);
@@ -1085,6 +1023,7 @@ gtk_tooltip_position (GtkTooltip *tooltip,
   GdkRectangle monitor;
   guint cursor_size;
   GdkRectangle bounds;
+  GtkBorder border;
 
 #define MAX_DISTANCE 32
 
@@ -1095,8 +1034,10 @@ gtk_tooltip_position (GtkTooltip *tooltip,
 
   screen = gtk_widget_get_screen (new_tooltip_widget);
 
-  width = gtk_widget_get_allocated_width (GTK_WIDGET (tooltip->current_window));
-  height = gtk_widget_get_allocated_height (GTK_WIDGET (tooltip->current_window));
+  _gtk_window_get_shadow_width (GTK_WINDOW (tooltip->current_window), &border);
+
+  width = gtk_widget_get_allocated_width (GTK_WIDGET (tooltip->current_window)) - border.left - border.right;
+  height = gtk_widget_get_allocated_height (GTK_WIDGET (tooltip->current_window)) - border.top - border.bottom;
 
   monitor_num = gdk_screen_get_monitor_at_point (screen,
                                                  tooltip->last_x,
@@ -1204,30 +1145,42 @@ gtk_tooltip_position (GtkTooltip *tooltip,
 
 found:
   /* Show it */
-  if (tooltip->current_window)
+  if (x + width > monitor.x + monitor.width)
+    x -= x - (monitor.x + monitor.width) + width;
+  else if (x < monitor.x)
+    x = monitor.x;
+
+  if (y + height > monitor.y + monitor.height)
+    y -= y - (monitor.y + monitor.height) + height;
+  else if (y < monitor.y)
+    y = monitor.y;
+
+  if (!tooltip->keyboard_mode_enabled)
     {
-      if (x + width > monitor.x + monitor.width)
-        x -= x - (monitor.x + monitor.width) + width;
-      else if (x < monitor.x)
-        x = monitor.x;
-
-      if (y + height > monitor.y + monitor.height)
-        y -= y - (monitor.y + monitor.height) + height;
-      else if (y < monitor.y)
-        y = monitor.y;
-
-      if (!tooltip->keyboard_mode_enabled)
-        {
-          /* don't pop up under the pointer */
-          if (x <= tooltip->last_x && tooltip->last_x < x + width &&
-              y <= tooltip->last_y && tooltip->last_y < y + height)
-            y = tooltip->last_y - height - 2;
-        }
+      /* don't pop up under the pointer */
+      if (x <= tooltip->last_x && tooltip->last_x < x + width &&
+          y <= tooltip->last_y && tooltip->last_y < y + height)
+        y = tooltip->last_y - height - 2;
+    }
 
 #ifdef GDK_WINDOWING_WAYLAND
-      /* set the transient parent on the tooltip when running with the Wayland
+  /* set the transient parent on the tooltip when running with the Wayland
+   * backend to allow correct positioning of the tooltip windows
+   */
+  if (GDK_IS_WAYLAND_DISPLAY (display))
+    {
+      GtkWidget *toplevel;
+
+      toplevel = gtk_widget_get_toplevel (tooltip->tooltip_widget);
+      if (GTK_IS_WINDOW (toplevel))
+        gtk_window_set_transient_for (GTK_WINDOW (tooltip->current_window),
+                                      GTK_WINDOW (toplevel));
+    }
+#endif
+#ifdef GDK_WINDOWING_MIR
+      /* Set the transient parent on the tooltip when running with the Mir
        * backend to allow correct positioning of the tooltip windows */
-      if (GDK_IS_WAYLAND_DISPLAY (display))
+      if (GDK_IS_MIR_DISPLAY (display))
         {
           GtkWidget *toplevel;
 
@@ -1238,9 +1191,11 @@ found:
         }
 #endif
 
-      gtk_window_move (GTK_WINDOW (tooltip->current_window), x, y);
-      gtk_widget_show (GTK_WIDGET (tooltip->current_window));
-    }
+  x -= border.left;
+  y -= border.top;
+
+  gtk_window_move (GTK_WINDOW (tooltip->current_window), x, y);
+  gtk_widget_show (GTK_WIDGET (tooltip->current_window));
 }
 
 static void
@@ -1252,7 +1207,6 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
   GdkWindow *window;
   GtkWidget *tooltip_widget;
   GtkTooltip *tooltip;
-  gboolean has_tooltip;
   gboolean return_value = FALSE;
 
   tooltip = g_object_get_data (G_OBJECT (display),
@@ -1287,8 +1241,6 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
   if (!tooltip_widget)
     return;
 
-  g_object_get (tooltip_widget, "has-tooltip", &has_tooltip, NULL);
-
   return_value = gtk_tooltip_run_requery (&tooltip_widget, tooltip, &x, &y);
   if (!return_value)
     return;
@@ -1550,7 +1502,6 @@ _gtk_tooltip_toggle_keyboard_mode (GtkWidget *widget)
 void
 _gtk_tooltip_hide (GtkWidget *widget)
 {
-  GtkWidget *toplevel;
   GdkDisplay *display;
   GtkTooltip *tooltip;
 
@@ -1561,10 +1512,7 @@ _gtk_tooltip_hide (GtkWidget *widget)
   if (!tooltip || !GTK_TOOLTIP_VISIBLE (tooltip) || !tooltip->tooltip_widget)
     return;
 
-  toplevel = gtk_widget_get_toplevel (widget);
-
-  if (widget == tooltip->tooltip_widget
-      || gtk_widget_get_window (toplevel) == tooltip->toplevel_window)
+  if (widget == tooltip->tooltip_widget)
     gtk_tooltip_hide_tooltip (tooltip);
 }
 
@@ -1630,13 +1578,17 @@ _gtk_tooltip_handle_event (GdkEvent *event)
 
 #ifdef DEBUG_TOOLTIP
   if (has_tooltip_widget)
+    {
+    GtkAllocation allocation;
+    gtk_widget_get_allocation (has_tooltip_widget, &allocation);
     g_print ("%p (%s) at (%d, %d) %dx%d     pointer: (%d, %d)\n",
 	     has_tooltip_widget, gtk_widget_get_name (has_tooltip_widget),
-	     has_tooltip_widget->allocation.x,
-	     has_tooltip_widget->allocation.y,
-	     has_tooltip_widget->allocation.width,
-	     has_tooltip_widget->allocation.height,
+	     allocation.x,
+	     allocation.y,
+	     allocation.width,
+	     allocation.height,
 	     x, y);
+    }
 #endif /* DEBUG_TOOLTIP */
 
   /* Always poll for a next motion event */
